# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from abc import ABC, abstractmethod
from collections import defaultdict
from collections.abc import Callable, Mapping, Sequence
from random import randrange
from typing import (
    Any,
)

from opentelemetry import trace
from opentelemetry.context import Context
from opentelemetry.trace.span import INVALID_SPAN
from opentelemetry.util.types import Attributes

from .exemplar import Exemplar


class ExemplarReservoir(ABC):
    """ExemplarReservoir provide a method to offer measurements to the reservoir
    and another to collect accumulated Exemplars.

    Note:
        The constructor MUST accept ``**kwargs`` that may be set from aggregation
        parameters.

    Reference:
        https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/metrics/sdk.md#exemplarreservoir
    """

    @abstractmethod
    def offer(
        self,
        value: int | float,
        time_unix_nano: int,
        attributes: Attributes,
        context: Context,
    ) -> None:
        """Offers a measurement to be sampled.

        Args:
            value: Measured value
            time_unix_nano: Measurement instant
            attributes: Measurement attributes
            context: Measurement context
        """
        raise NotImplementedError("ExemplarReservoir.offer is not implemented")

    @abstractmethod
    def collect(self, point_attributes: Attributes) -> list[Exemplar]:
        """Returns accumulated Exemplars and also resets the reservoir for the next
        sampling period

        Args:
            point_attributes: The attributes associated with metric point.

        Returns:
            a list of ``opentelemetry.sdk.metrics._internal.exemplar.exemplar.Exemplar`` s. Returned
            exemplars contain the attributes that were filtered out by the aggregator,
            but recorded alongside the original measurement.
        """
        raise NotImplementedError(
            "ExemplarReservoir.collect is not implemented"
        )


class ExemplarBucket:
    def __init__(self) -> None:
        self.__value: int | float = 0
        self.__attributes: Attributes = None
        self.__time_unix_nano: int = 0
        self.__span_id: int | None = None
        self.__trace_id: int | None = None
        self.__offered: bool = False

    def offer(
        self,
        value: int | float,
        time_unix_nano: int,
        attributes: Attributes,
        context: Context,
    ) -> None:
        """Offers a measurement to be sampled.

        Args:
            value: Measured value
            time_unix_nano: Measurement instant
            attributes: Measurement attributes
            context: Measurement context
        """
        self.__value = value
        self.__time_unix_nano = time_unix_nano
        self.__attributes = attributes
        span = trace.get_current_span(context)
        if span != INVALID_SPAN:
            span_context = span.get_span_context()
            self.__span_id = span_context.span_id
            self.__trace_id = span_context.trace_id

        self.__offered = True

    def collect(self, point_attributes: Attributes) -> Exemplar | None:
        """May return an Exemplar and resets the bucket for the next sampling period."""
        if not self.__offered:
            return None

        # filters out attributes from the measurement that are already included in the metric data point
        # See the specification for more details:
        # https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/metrics/sdk.md#exemplar
        filtered_attributes = (
            {
                k: v
                for k, v in self.__attributes.items()
                if k not in point_attributes
            }
            if self.__attributes
            else None
        )

        exemplar = Exemplar(
            filtered_attributes,
            self.__value,
            self.__time_unix_nano,
            self.__span_id,
            self.__trace_id,
        )
        self.__reset()
        return exemplar

    def __reset(self) -> None:
        """Reset the bucket state after a collection cycle."""
        self.__value = 0
        self.__attributes = {}
        self.__time_unix_nano = 0
        self.__span_id = None
        self.__trace_id = None
        self.__offered = False


class BucketIndexError(ValueError):
    """An exception raised when the bucket index cannot be found."""


class FixedSizeExemplarReservoirABC(ExemplarReservoir):
    """Abstract class for a reservoir with fixed size."""

    def __init__(self, size: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self._size: int = size
        self._reservoir_storage: Mapping[int, ExemplarBucket] = defaultdict(
            ExemplarBucket
        )

    def collect(self, point_attributes: Attributes) -> list[Exemplar]:
        """Returns accumulated Exemplars and also resets the reservoir for the next
        sampling period

        Args:
            point_attributes: The attributes associated with metric point.

        Returns:
            a list of ``opentelemetry.sdk.metrics._internal.exemplar.exemplar.Exemplar`` s. Returned
            exemplars contain the attributes that were filtered out by the aggregator,
            but recorded alongside the original measurement.
        """
        exemplars = [
            e
            for e in (
                bucket.collect(point_attributes)
                for _, bucket in sorted(self._reservoir_storage.items())
            )
            if e is not None
        ]
        self._reset()
        return exemplars

    def offer(
        self,
        value: int | float,
        time_unix_nano: int,
        attributes: Attributes,
        context: Context,
    ) -> None:
        """Offers a measurement to be sampled.

        Args:
            value: Measured value
            time_unix_nano: Measurement instant
            attributes: Measurement attributes
            context: Measurement context
        """
        try:
            index = self._find_bucket_index(
                value, time_unix_nano, attributes, context
            )

            self._reservoir_storage[index].offer(
                value, time_unix_nano, attributes, context
            )
        except BucketIndexError:
            # Ignore invalid bucket index
            pass

    @abstractmethod
    def _find_bucket_index(
        self,
        value: int | float,
        time_unix_nano: int,
        attributes: Attributes,
        context: Context,
    ) -> int:
        """Determines the bucket index for the given measurement.

        It should be implemented by subclasses based on specific strategies.

        Args:
            value: Measured value
            time_unix_nano: Measurement instant
            attributes: Measurement attributes
            context: Measurement context

        Returns:
            The bucket index

        Raises:
            BucketIndexError: If no bucket index can be found.
        """

    def _reset(self) -> None:
        """Reset the reservoir by resetting any stateful logic after a collection cycle."""


class SimpleFixedSizeExemplarReservoir(FixedSizeExemplarReservoirABC):
    """This reservoir uses an uniformly-weighted sampling algorithm based on the number
    of samples the reservoir has seen so far to determine if the offered measurements
    should be sampled.

    Reference:
        https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/metrics/sdk.md#simplefixedsizeexemplarreservoir
    """

    def __init__(self, size: int = 1, **kwargs) -> None:
        super().__init__(size, **kwargs)
        self._measurements_seen: int = 0

    def _reset(self) -> None:
        super()._reset()
        self._measurements_seen = 0

    def _find_bucket_index(
        self,
        value: int | float,
        time_unix_nano: int,
        attributes: Attributes,
        context: Context,
    ) -> int:
        self._measurements_seen += 1
        if self._measurements_seen < self._size:
            return self._measurements_seen - 1

        index = randrange(0, self._measurements_seen)
        if index < self._size:
            return index

        raise BucketIndexError("Unable to find the bucket index.")


class AlignedHistogramBucketExemplarReservoir(FixedSizeExemplarReservoirABC):
    """This Exemplar reservoir takes a configuration parameter that is the
    configuration of a Histogram. This implementation keeps the last seen measurement
    that falls within a histogram bucket.

    Reference:
        https://github.com/open-telemetry/opentelemetry-specification/blob/main/specification/metrics/sdk.md#alignedhistogrambucketexemplarreservoir
    """

    def __init__(self, boundaries: Sequence[float], **kwargs) -> None:
        super().__init__(len(boundaries) + 1, **kwargs)
        self._boundaries: Sequence[float] = boundaries

    def offer(
        self,
        value: int | float,
        time_unix_nano: int,
        attributes: Attributes,
        context: Context,
    ) -> None:
        """Offers a measurement to be sampled."""
        index = self._find_bucket_index(
            value, time_unix_nano, attributes, context
        )
        self._reservoir_storage[index].offer(
            value, time_unix_nano, attributes, context
        )

    def _find_bucket_index(
        self,
        value: int | float,
        time_unix_nano: int,
        attributes: Attributes,
        context: Context,
    ) -> int:
        for index, boundary in enumerate(self._boundaries):
            if value <= boundary:
                return index
        return len(self._boundaries)


# ExemplarReservoirBuilder: A callable that takes Aggregation parameters
# (e.g. boundaries for _ExplicitBucketHistogramAggregation) and returns
# an ExemplarReservoir instance.
ExemplarReservoirBuilder = Callable[[dict[str, Any]], ExemplarReservoir]



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from time import time, time_ns
from unittest import TestCase

from opentelemetry.shim.opentracing_shim.util import (
    DEFAULT_EVENT_NAME,
    event_name_from_kv,
    time_seconds_from_ns,
    time_seconds_to_ns,
)


class TestUtil(TestCase):
    def test_event_name_from_kv(self):
        # Test basic behavior.
        event_name = "send HTTP request"
        res = event_name_from_kv({"event": event_name, "foo": "bar"})
        self.assertEqual(res, event_name)

        # Test None.
        res = event_name_from_kv(None)
        self.assertEqual(res, DEFAULT_EVENT_NAME)

        # Test empty dict.
        res = event_name_from_kv({})
        self.assertEqual(res, DEFAULT_EVENT_NAME)

        # Test missing `event` field.
        res = event_name_from_kv({"foo": "bar"})
        self.assertEqual(res, DEFAULT_EVENT_NAME)

    def test_time_seconds_to_ns(self):
        time_seconds = time()
        result = time_seconds_to_ns(time_seconds)

        self.assertEqual(result, int(time_seconds * 1e9))

    def test_time_seconds_from_ns(self):
        time_nanoseconds = time_ns()
        result = time_seconds_from_ns(time_nanoseconds)

        self.assertEqual(result, time_nanoseconds / 1e9)

    def test_time_conversion_precision(self):
        """Verify time conversion from seconds to nanoseconds and vice versa is
        accurate enough.
        """

        time_seconds = 1570484241.9501917
        time_nanoseconds = time_seconds_to_ns(time_seconds)
        result = time_seconds_from_ns(time_nanoseconds)

        # Tolerate inaccuracies of less than a microsecond.
        # TODO: Put a link to an explanation in the docs.
        # TODO: This seems to work consistently, but we should find out the
        # biggest possible loss of precision.
        self.assertAlmostEqual(result, time_seconds, places=6)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# pylint: disable=protected-access

from unittest import TestCase
from unittest.mock import Mock

from opentelemetry.sdk.metrics.view import View


class TestView(TestCase):
    def test_required_instrument_criteria(self):
        with self.assertRaises(Exception):
            View()

    def test_instrument_type(self):
        self.assertTrue(View(instrument_type=Mock)._match(Mock()))

    def test_instrument_name(self):
        mock_instrument = Mock()
        mock_instrument.configure_mock(**{"name": "instrument_name"})

        self.assertTrue(
            View(instrument_name="instrument_name")._match(mock_instrument)
        )

    def test_instrument_unit(self):
        mock_instrument = Mock()
        mock_instrument.configure_mock(**{"unit": "instrument_unit"})

        self.assertTrue(
            View(instrument_unit="instrument_unit")._match(mock_instrument)
        )

    def test_meter_name(self):
        self.assertTrue(
            View(meter_name="meter_name")._match(
                Mock(**{"instrumentation_scope.name": "meter_name"})
            )
        )

    def test_meter_version(self):
        self.assertTrue(
            View(meter_version="meter_version")._match(
                Mock(**{"instrumentation_scope.version": "meter_version"})
            )
        )

    def test_meter_schema_url(self):
        self.assertTrue(
            View(meter_schema_url="meter_schema_url")._match(
                Mock(
                    **{"instrumentation_scope.schema_url": "meter_schema_url"}
                )
            )
        )
        self.assertFalse(
            View(meter_schema_url="meter_schema_url")._match(
                Mock(
                    **{
                        "instrumentation_scope.schema_url": "meter_schema_urlabc"
                    }
                )
            )
        )
        self.assertTrue(
            View(meter_schema_url="meter_schema_url")._match(
                Mock(
                    **{"instrumentation_scope.schema_url": "meter_schema_url"}
                )
            )
        )

    def test_additive_criteria(self):
        view = View(
            meter_name="meter_name",
            meter_version="meter_version",
            meter_schema_url="meter_schema_url",
        )

        self.assertTrue(
            view._match(
                Mock(
                    **{
                        "instrumentation_scope.name": "meter_name",
                        "instrumentation_scope.version": "meter_version",
                        "instrumentation_scope.schema_url": "meter_schema_url",
                    }
                )
            )
        )
        self.assertFalse(
            view._match(
                Mock(
                    **{
                        "instrumentation_scope.name": "meter_name",
                        "instrumentation_scope.version": "meter_version",
                        "instrumentation_scope.schema_url": "meter_schema_vrl",
                    }
                )
            )
        )

    def test_view_name(self):
        with self.assertRaises(Exception):
            View(name="name", instrument_name="instrument_name*")



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import pytest

from opentelemetry._logs import SeverityNumber
from opentelemetry.sdk._logs import LoggerProvider
from opentelemetry.sdk._logs.export import (
    BatchLogRecordProcessor,
    InMemoryLogRecordExporter,
    SimpleLogRecordProcessor,
)
from opentelemetry.sdk.resources import Resource

resource = Resource(
    {
        "service.name": "A123456789",
        "service.version": "1.34567890",
        "service.instance.id": "123ab456-a123-12ab-12ab-12340a1abc12",
    }
)

simple_exporter = InMemoryLogRecordExporter()
simple_provider = LoggerProvider(resource=resource)
simple_provider.add_log_record_processor(
    SimpleLogRecordProcessor(simple_exporter)
)
simple_logger = simple_provider.get_logger("simple_logger")

batch_exporter = InMemoryLogRecordExporter()
batch_provider = LoggerProvider(resource=resource)
batch_provider.add_log_record_processor(
    BatchLogRecordProcessor(batch_exporter)
)
batch_logger = batch_provider.get_logger("batch_logger")


@pytest.mark.parametrize("num_attributes", [0, 1, 3, 5, 10])
def test_simple_log_record_processor(benchmark, num_attributes):
    attributes = {f"key{i}": f"value{i}" for i in range(num_attributes)}

    def benchmark_emit():
        simple_logger.emit(
            severity_number=SeverityNumber.INFO,
            body="benchmark log message",
            attributes=attributes,
            event_name="test.event",
        )

    benchmark(benchmark_emit)


@pytest.mark.parametrize("num_attributes", [0, 1, 3, 5, 10])
def test_batch_log_record_processor(benchmark, num_attributes):
    attributes = {f"key{i}": f"value{i}" for i in range(num_attributes)}

    def benchmark_emit():
        batch_logger.emit(
            severity_number=SeverityNumber.INFO,
            body="benchmark log message",
            attributes=attributes,
            event_name="test.event",
        )

    benchmark(benchmark_emit)


def test_get_logger(benchmark):
    def benchmark_get_logger():
        simple_provider.get_logger(
            "test_logger",
            version="1.0.0",
            schema_url="https://opentelemetry.io/schemas/1.38.0",
        )

    benchmark(benchmark_get_logger)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# Tests access private members of SDK classes to assert correct configuration.
# pylint: disable=protected-access

import unittest

from opentelemetry.configuration._meter_provider import (
    create_meter_provider,
)
from opentelemetry.configuration.models import (
    ExemplarFilter as ExemplarFilterConfig,
)
from opentelemetry.configuration.models import (
    MeterProvider as MeterProviderConfig,
)
from opentelemetry.sdk.metrics import (
    AlwaysOffExemplarFilter,
    AlwaysOnExemplarFilter,
    TraceBasedExemplarFilter,
)


class TestExemplarFilter(unittest.TestCase):
    @staticmethod
    def _make_config(exemplar_filter):
        return MeterProviderConfig(readers=[], exemplar_filter=exemplar_filter)

    def test_always_on(self):
        provider = create_meter_provider(
            self._make_config(ExemplarFilterConfig.always_on)
        )
        self.assertIsInstance(
            provider._sdk_config.exemplar_filter, AlwaysOnExemplarFilter
        )

    def test_always_off(self):
        provider = create_meter_provider(
            self._make_config(ExemplarFilterConfig.always_off)
        )
        self.assertIsInstance(
            provider._sdk_config.exemplar_filter, AlwaysOffExemplarFilter
        )

    def test_trace_based(self):
        provider = create_meter_provider(
            self._make_config(ExemplarFilterConfig.trace_based)
        )
        self.assertIsInstance(
            provider._sdk_config.exemplar_filter, TraceBasedExemplarFilter
        )

    def test_absent_defaults_to_trace_based(self):
        provider = create_meter_provider(MeterProviderConfig(readers=[]))
        self.assertIsInstance(
            provider._sdk_config.exemplar_filter, TraceBasedExemplarFilter
        )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import math
from collections.abc import Iterable

from opentelemetry.metrics import (
    CallbackOptions,
    Observation,
    get_meter_provider,
    set_meter_provider,
)
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import (
    ConsoleMetricExporter,
    PeriodicExportingMetricReader,
)

temperature = 0.0
humidity = 0.0


# Function called by the gauge to read the temperature
def read_temperature(options: CallbackOptions) -> Iterable[Observation]:
    global temperature
    yield Observation(value=temperature, attributes={"room": "living-room"})


# Function called by the gauge to read the humidity
def read_humidity(options: CallbackOptions) -> Iterable[Observation]:
    global humidity
    yield Observation(value=humidity, attributes={"room": "living-room"})


# Use console exporter for the example
exporter = ConsoleMetricExporter()

# The PeriodicExportingMetricReader If the time interval is set to math.inf
# the reader will not invoke periodic collection
reader = PeriodicExportingMetricReader(
    exporter,
    export_interval_millis=math.inf,
)

provider = MeterProvider(metric_readers=[reader])
set_meter_provider(provider)

meter = get_meter_provider().get_meter("synchronous_read", "0.1.2")

gauge = meter.create_observable_gauge(
    name="synchronous_gauge_temperature",
    description="Gauge value captured synchronously",
    callbacks=[read_temperature],
)

# Simulate synchronous reading of temperature
print("--- Simulating synchronous reading of temperature ---", flush=True)
temperature = 25.0
reader.collect()
# Note: The reader will only collect the last value before `collect` is called
print("--- Last value only ---", flush=True)
temperature = 30.0
temperature = 35.0
reader.collect()
# Invoking `collect` will read all measurements assigned to the reader
gauge2 = meter.create_observable_gauge(
    name="synchronous_gauge_humidity",
    description="Gauge value captured synchronously",
    callbacks=[read_humidity],
)
print("--- Multiple Measurements ---", flush=True)
temperature = 20.0
humidity = 50.0
reader.collect()
# Invoking `force_flush` will read all measurements assigned to the reader
print("--- Invoking force_flush ---", flush=True)
provider.force_flush()



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# pylint: disable=no-self-use

from logging import WARNING

# from time import time_ns
from unittest import TestCase
from unittest.mock import Mock, patch

from opentelemetry.context import Context
from opentelemetry.metrics import Observation
from opentelemetry.metrics._internal.instrument import CallbackOptions
from opentelemetry.sdk.metrics import (
    Counter,
    Histogram,
    ObservableCounter,
    ObservableGauge,
    ObservableUpDownCounter,
    UpDownCounter,
)
from opentelemetry.sdk.metrics import _Gauge as _SDKGauge
from opentelemetry.sdk.metrics._internal.instrument import (
    _Counter,
    _Gauge,
    _Histogram,
    _ObservableCounter,
    _ObservableGauge,
    _ObservableUpDownCounter,
    _UpDownCounter,
)
from opentelemetry.sdk.metrics._internal.measurement import Measurement


class TestCounter(TestCase):
    def testname(self):
        self.assertEqual(_Counter("name", Mock(), Mock()).name, "name")
        self.assertEqual(_Counter("Name", Mock(), Mock()).name, "name")

    def test_add(self):
        mc = Mock()
        counter = _Counter("name", Mock(), mc)
        counter.add(1.0)
        mc.consume_measurement.assert_called_once()

    def test_add_non_monotonic(self):
        mc = Mock()
        counter = _Counter("name", Mock(), mc)
        with self.assertLogs(level=WARNING):
            counter.add(-1.0)
        mc.consume_measurement.assert_not_called()

    def test_add_nan(self):
        mc = Mock()
        counter = _Counter("name", Mock(), mc)
        with self.assertLogs(level=WARNING):
            counter.add(float("nan"))
        mc.consume_measurement.assert_not_called()

    def test_add_inf(self):
        mc = Mock()
        counter = _Counter("name", Mock(), mc)
        with self.assertLogs(level=WARNING):
            counter.add(float("inf"))
        mc.consume_measurement.assert_not_called()

    def test_disallow_direct_counter_creation(self):
        with self.assertRaises(TypeError):
            # pylint: disable=abstract-class-instantiated
            Counter("name", Mock(), Mock())


class TestUpDownCounter(TestCase):
    def test_add(self):
        mc = Mock()
        counter = _UpDownCounter("name", Mock(), mc)
        counter.add(1.0)
        mc.consume_measurement.assert_called_once()

    def test_add_non_monotonic(self):
        mc = Mock()
        counter = _UpDownCounter("name", Mock(), mc)
        counter.add(-1.0)
        mc.consume_measurement.assert_called_once()

    def test_add_nan(self):
        mc = Mock()
        counter = _UpDownCounter("name", Mock(), mc)
        with self.assertLogs(level=WARNING):
            counter.add(float("nan"))
        mc.consume_measurement.assert_not_called()

    def test_add_inf(self):
        mc = Mock()
        counter = _UpDownCounter("name", Mock(), mc)
        with self.assertLogs(level=WARNING):
            counter.add(float("inf"))
        mc.consume_measurement.assert_not_called()

    def test_disallow_direct_up_down_counter_creation(self):
        with self.assertRaises(TypeError):
            # pylint: disable=abstract-class-instantiated
            UpDownCounter("name", Mock(), Mock())


TEST_ATTRIBUTES = {"foo": "bar"}
TEST_CONTEXT = Context()
TEST_TIMESTAMP = 1_000_000_000


def callable_callback_0(options: CallbackOptions):
    return [
        Observation(1, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
        Observation(2, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
        Observation(3, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
    ]


def callable_callback_1(options: CallbackOptions):
    return [
        Observation(4, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
        Observation(5, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
        Observation(6, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
    ]


def generator_callback_0():
    options = yield
    assert isinstance(options, CallbackOptions)
    options = yield [
        Observation(1, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
        Observation(2, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
        Observation(3, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
    ]
    assert isinstance(options, CallbackOptions)


def generator_callback_1():
    options = yield
    assert isinstance(options, CallbackOptions)
    options = yield [
        Observation(4, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
        Observation(5, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
        Observation(6, attributes=TEST_ATTRIBUTES, context=TEST_CONTEXT),
    ]
    assert isinstance(options, CallbackOptions)


@patch(
    "opentelemetry.sdk.metrics._internal.instrument.time_ns",
    Mock(return_value=TEST_TIMESTAMP),
)
class TestObservableGauge(TestCase):
    def testname(self):
        self.assertEqual(_ObservableGauge("name", Mock(), Mock()).name, "name")
        self.assertEqual(_ObservableGauge("Name", Mock(), Mock()).name, "name")

    def test_callable_callback_0(self):
        observable_gauge = _ObservableGauge(
            "name", Mock(), Mock(), [callable_callback_0]
        )

        assert list(observable_gauge.callback(CallbackOptions())) == (
            [
                Measurement(
                    1,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    2,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    3,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
            ]
        )

    def test_callable_multiple_callable_callback(self):
        observable_gauge = _ObservableGauge(
            "name", Mock(), Mock(), [callable_callback_0, callable_callback_1]
        )

        self.assertEqual(
            list(observable_gauge.callback(CallbackOptions())),
            [
                Measurement(
                    1,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    2,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    3,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    4,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    5,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    6,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
            ],
        )

    def test_generator_callback_0(self):
        observable_gauge = _ObservableGauge(
            "name", Mock(), Mock(), [generator_callback_0()]
        )

        self.assertEqual(
            list(observable_gauge.callback(CallbackOptions())),
            [
                Measurement(
                    1,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    2,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    3,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
            ],
        )

    def test_generator_multiple_generator_callback(self):
        observable_gauge = _ObservableGauge(
            "name",
            Mock(),
            Mock(),
            callbacks=[generator_callback_0(), generator_callback_1()],
        )

        self.assertEqual(
            list(observable_gauge.callback(CallbackOptions())),
            [
                Measurement(
                    1,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    2,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    3,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    4,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    5,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    6,
                    TEST_TIMESTAMP,
                    instrument=observable_gauge,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
            ],
        )

    def test_nan_callback_value_is_dropped(self):
        def nan_callback(options: CallbackOptions):
            return [
                Observation(float("nan"), attributes=TEST_ATTRIBUTES),
                Observation(1, attributes=TEST_ATTRIBUTES),
            ]

        observable_gauge = _ObservableGauge(
            "name", Mock(), Mock(), [nan_callback]
        )
        with self.assertLogs(level=WARNING):
            measurements = list(observable_gauge.callback(CallbackOptions()))
        self.assertEqual(len(measurements), 1)
        self.assertEqual(measurements[0].value, 1)

    def test_inf_callback_value_is_dropped(self):
        def inf_callback(options: CallbackOptions):
            return [
                Observation(float("inf"), attributes=TEST_ATTRIBUTES),
                Observation(1, attributes=TEST_ATTRIBUTES),
            ]

        observable_gauge = _ObservableGauge(
            "name", Mock(), Mock(), [inf_callback]
        )
        with self.assertLogs(level=WARNING):
            measurements = list(observable_gauge.callback(CallbackOptions()))
        self.assertEqual(len(measurements), 1)
        self.assertEqual(measurements[0].value, 1)

    def test_disallow_direct_observable_gauge_creation(self):
        with self.assertRaises(TypeError):
            # pylint: disable=abstract-class-instantiated
            ObservableGauge("name", Mock(), Mock())


@patch(
    "opentelemetry.sdk.metrics._internal.instrument.time_ns",
    Mock(return_value=TEST_TIMESTAMP),
)
class TestObservableCounter(TestCase):
    def test_callable_callback_0(self):
        observable_counter = _ObservableCounter(
            "name", Mock(), Mock(), [callable_callback_0]
        )

        self.assertEqual(
            list(observable_counter.callback(CallbackOptions())),
            [
                Measurement(
                    1,
                    TEST_TIMESTAMP,
                    instrument=observable_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    2,
                    TEST_TIMESTAMP,
                    instrument=observable_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    3,
                    TEST_TIMESTAMP,
                    instrument=observable_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
            ],
        )

    def test_generator_callback_0(self):
        observable_counter = _ObservableCounter(
            "name", Mock(), Mock(), [generator_callback_0()]
        )

        self.assertEqual(
            list(observable_counter.callback(CallbackOptions())),
            [
                Measurement(
                    1,
                    TEST_TIMESTAMP,
                    instrument=observable_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    2,
                    TEST_TIMESTAMP,
                    instrument=observable_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    3,
                    TEST_TIMESTAMP,
                    instrument=observable_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
            ],
        )

    def test_disallow_direct_observable_counter_creation(self):
        with self.assertRaises(TypeError):
            # pylint: disable=abstract-class-instantiated
            ObservableCounter("name", Mock(), Mock())


class TestGauge(TestCase):
    def testname(self):
        self.assertEqual(_Gauge("name", Mock(), Mock()).name, "name")
        self.assertEqual(_Gauge("Name", Mock(), Mock()).name, "name")

    def test_set(self):
        mc = Mock()
        gauge = _Gauge("name", Mock(), mc)
        gauge.set(1.0)
        mc.consume_measurement.assert_called_once()

    def test_set_nan(self):
        mc = Mock()
        gauge = _Gauge("name", Mock(), mc)
        with self.assertLogs(level=WARNING):
            gauge.set(float("nan"))
        mc.consume_measurement.assert_not_called()

    def test_set_inf(self):
        mc = Mock()
        gauge = _Gauge("name", Mock(), mc)
        with self.assertLogs(level=WARNING):
            gauge.set(float("inf"))
        mc.consume_measurement.assert_not_called()

    def test_disallow_direct_counter_creation(self):
        with self.assertRaises(TypeError):
            # pylint: disable=abstract-class-instantiated
            _SDKGauge("name", Mock(), Mock())


@patch(
    "opentelemetry.sdk.metrics._internal.instrument.time_ns",
    Mock(return_value=TEST_TIMESTAMP),
)
class TestObservableUpDownCounter(TestCase):
    def test_callable_callback_0(self):
        observable_up_down_counter = _ObservableUpDownCounter(
            "name", Mock(), Mock(), [callable_callback_0]
        )

        self.assertEqual(
            list(observable_up_down_counter.callback(CallbackOptions())),
            [
                Measurement(
                    1,
                    TEST_TIMESTAMP,
                    instrument=observable_up_down_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    2,
                    TEST_TIMESTAMP,
                    instrument=observable_up_down_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    3,
                    TEST_TIMESTAMP,
                    instrument=observable_up_down_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
            ],
        )

    def test_generator_callback_0(self):
        observable_up_down_counter = _ObservableUpDownCounter(
            "name", Mock(), Mock(), [generator_callback_0()]
        )

        self.assertEqual(
            list(observable_up_down_counter.callback(CallbackOptions())),
            [
                Measurement(
                    1,
                    TEST_TIMESTAMP,
                    instrument=observable_up_down_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    2,
                    TEST_TIMESTAMP,
                    instrument=observable_up_down_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
                Measurement(
                    3,
                    TEST_TIMESTAMP,
                    instrument=observable_up_down_counter,
                    context=TEST_CONTEXT,
                    attributes=TEST_ATTRIBUTES,
                ),
            ],
        )

    def test_disallow_direct_observable_up_down_counter_creation(self):
        with self.assertRaises(TypeError):
            # pylint: disable=abstract-class-instantiated
            ObservableUpDownCounter("name", Mock(), Mock())


class TestHistogram(TestCase):
    def test_record(self):
        mc = Mock()
        hist = _Histogram("name", Mock(), mc)
        hist.record(1.0)
        mc.consume_measurement.assert_called_once()

    def test_record_non_monotonic(self):
        mc = Mock()
        hist = _Histogram("name", Mock(), mc)
        with self.assertLogs(level=WARNING):
            hist.record(-1.0)
        mc.consume_measurement.assert_not_called()

    def test_record_nan(self):
        mc = Mock()
        hist = _Histogram("name", Mock(), mc)
        with self.assertLogs(level=WARNING):
            hist.record(float("nan"))
        mc.consume_measurement.assert_not_called()

    def test_record_inf(self):
        mc = Mock()
        hist = _Histogram("name", Mock(), mc)
        with self.assertLogs(level=WARNING):
            hist.record(float("inf"))
        mc.consume_measurement.assert_not_called()

    def test_disallow_direct_histogram_creation(self):
        with self.assertRaises(TypeError):
            # pylint: disable=abstract-class-instantiated
            Histogram("name", Mock(), Mock())



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2019, OpenTelemetry Authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import opentelemetry.proto.common.v1.common_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class Resource(google.protobuf.message.Message):
    """Resource information."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ATTRIBUTES_FIELD_NUMBER: builtins.int
    DROPPED_ATTRIBUTES_COUNT_FIELD_NUMBER: builtins.int
    ENTITY_REFS_FIELD_NUMBER: builtins.int
    @property
    def attributes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[opentelemetry.proto.common.v1.common_pb2.KeyValue]:
        """Set of attributes that describe the resource.
        Attribute keys MUST be unique (it is not allowed to have more than one
        attribute with the same key).
        The behavior of software that receives duplicated keys can be unpredictable.
        """
    dropped_attributes_count: builtins.int
    """The number of dropped attributes. If the value is 0, then
    no attributes were dropped.
    """
    @property
    def entity_refs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[opentelemetry.proto.common.v1.common_pb2.EntityRef]:
        """Set of entities that participate in this Resource.

        Note: keys in the references MUST exist in attributes of this message.

        Status: [Development]
        """
    def __init__(
        self,
        *,
        attributes: collections.abc.Iterable[opentelemetry.proto.common.v1.common_pb2.KeyValue] | None = ...,
        dropped_attributes_count: builtins.int = ...,
        entity_refs: collections.abc.Iterable[opentelemetry.proto.common.v1.common_pb2.EntityRef] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["attributes", b"attributes", "dropped_attributes_count", b"dropped_attributes_count", "entity_refs", b"entity_refs"]) -> None: ...

global___Resource = Resource



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# AUTO-GENERATED from "opentelemetry/proto/logs/v1/logs.proto"
# DO NOT EDIT MANUALLY

from __future__ import annotations

import builtins
import dataclasses
import enum
import functools
import typing

_dataclass = functools.partial(dataclasses.dataclass, slots=True)

import opentelemetry.proto_json._json_codec
import opentelemetry.proto_json.common.v1.common
import opentelemetry.proto_json.resource.v1.resource


@typing.final
class SeverityNumber(enum.IntEnum):
    """
    Generated from protobuf enum SeverityNumber
    """

    SEVERITY_NUMBER_UNSPECIFIED = 0
    SEVERITY_NUMBER_TRACE = 1
    SEVERITY_NUMBER_TRACE2 = 2
    SEVERITY_NUMBER_TRACE3 = 3
    SEVERITY_NUMBER_TRACE4 = 4
    SEVERITY_NUMBER_DEBUG = 5
    SEVERITY_NUMBER_DEBUG2 = 6
    SEVERITY_NUMBER_DEBUG3 = 7
    SEVERITY_NUMBER_DEBUG4 = 8
    SEVERITY_NUMBER_INFO = 9
    SEVERITY_NUMBER_INFO2 = 10
    SEVERITY_NUMBER_INFO3 = 11
    SEVERITY_NUMBER_INFO4 = 12
    SEVERITY_NUMBER_WARN = 13
    SEVERITY_NUMBER_WARN2 = 14
    SEVERITY_NUMBER_WARN3 = 15
    SEVERITY_NUMBER_WARN4 = 16
    SEVERITY_NUMBER_ERROR = 17
    SEVERITY_NUMBER_ERROR2 = 18
    SEVERITY_NUMBER_ERROR3 = 19
    SEVERITY_NUMBER_ERROR4 = 20
    SEVERITY_NUMBER_FATAL = 21
    SEVERITY_NUMBER_FATAL2 = 22
    SEVERITY_NUMBER_FATAL3 = 23
    SEVERITY_NUMBER_FATAL4 = 24

@typing.final
class LogRecordFlags(enum.IntEnum):
    """
    Generated from protobuf enum LogRecordFlags
    """

    LOG_RECORD_FLAGS_DO_NOT_USE = 0
    LOG_RECORD_FLAGS_TRACE_FLAGS_MASK = 255

@typing.final
@_dataclass
class LogsData(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message LogsData
    """

    resource_logs: builtins.list[ResourceLogs] = dataclasses.field(default_factory=builtins.list)

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.resource_logs:
            _result["resourceLogs"] = opentelemetry.proto_json._json_codec.encode_repeated(self.resource_logs, lambda _v: _v.to_dict())
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "LogsData":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            LogsData instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("resourceLogs")) is not None:
            _args["resource_logs"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: ResourceLogs.from_dict(_v), "resource_logs")

        return cls(**_args)


@typing.final
@_dataclass
class ResourceLogs(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ResourceLogs
    """

    resource: opentelemetry.proto_json.resource.v1.resource.Resource | None = None
    scope_logs: builtins.list[ScopeLogs] = dataclasses.field(default_factory=builtins.list)
    schema_url: builtins.str | None = ""

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.resource:
            _result["resource"] = self.resource.to_dict()
        if self.scope_logs:
            _result["scopeLogs"] = opentelemetry.proto_json._json_codec.encode_repeated(self.scope_logs, lambda _v: _v.to_dict())
        if self.schema_url:
            _result["schemaUrl"] = self.schema_url
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ResourceLogs":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ResourceLogs instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("resource")) is not None:
            _args["resource"] = opentelemetry.proto_json.resource.v1.resource.Resource.from_dict(_value)
        if (_value := data.get("scopeLogs")) is not None:
            _args["scope_logs"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: ScopeLogs.from_dict(_v), "scope_logs")
        if (_value := data.get("schemaUrl")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "schema_url")
            _args["schema_url"] = _value

        return cls(**_args)


@typing.final
@_dataclass
class ScopeLogs(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ScopeLogs
    """

    scope: opentelemetry.proto_json.common.v1.common.InstrumentationScope | None = None
    log_records: builtins.list[LogRecord] = dataclasses.field(default_factory=builtins.list)
    schema_url: builtins.str | None = ""

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.scope:
            _result["scope"] = self.scope.to_dict()
        if self.log_records:
            _result["logRecords"] = opentelemetry.proto_json._json_codec.encode_repeated(self.log_records, lambda _v: _v.to_dict())
        if self.schema_url:
            _result["schemaUrl"] = self.schema_url
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ScopeLogs":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ScopeLogs instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("scope")) is not None:
            _args["scope"] = opentelemetry.proto_json.common.v1.common.InstrumentationScope.from_dict(_value)
        if (_value := data.get("logRecords")) is not None:
            _args["log_records"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: LogRecord.from_dict(_v), "log_records")
        if (_value := data.get("schemaUrl")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "schema_url")
            _args["schema_url"] = _value

        return cls(**_args)


@typing.final
@_dataclass
class LogRecord(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message LogRecord
    """

    time_unix_nano: builtins.int | None = 0
    observed_time_unix_nano: builtins.int | None = 0
    severity_number: SeverityNumber | builtins.int | None = 0
    severity_text: builtins.str | None = ""
    body: opentelemetry.proto_json.common.v1.common.AnyValue | None = None
    attributes: builtins.list[opentelemetry.proto_json.common.v1.common.KeyValue] = dataclasses.field(default_factory=builtins.list)
    dropped_attributes_count: builtins.int | None = 0
    flags: builtins.int | None = 0
    trace_id: builtins.bytes | None = b""
    span_id: builtins.bytes | None = b""
    event_name: builtins.str | None = ""

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.time_unix_nano:
            _result["timeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.time_unix_nano)
        if self.observed_time_unix_nano:
            _result["observedTimeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.observed_time_unix_nano)
        if self.severity_number:
            _result["severityNumber"] = builtins.int(self.severity_number)
        if self.severity_text:
            _result["severityText"] = self.severity_text
        if self.body:
            _result["body"] = self.body.to_dict()
        if self.attributes:
            _result["attributes"] = opentelemetry.proto_json._json_codec.encode_repeated(self.attributes, lambda _v: _v.to_dict())
        if self.dropped_attributes_count:
            _result["droppedAttributesCount"] = self.dropped_attributes_count
        if self.flags:
            _result["flags"] = self.flags
        if self.trace_id:
            _result["traceId"] = opentelemetry.proto_json._json_codec.encode_hex(self.trace_id)
        if self.span_id:
            _result["spanId"] = opentelemetry.proto_json._json_codec.encode_hex(self.span_id)
        if self.event_name:
            _result["eventName"] = self.event_name
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "LogRecord":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            LogRecord instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("timeUnixNano")) is not None:
            _args["time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "time_unix_nano")
        if (_value := data.get("observedTimeUnixNano")) is not None:
            _args["observed_time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "observed_time_unix_nano")
        if (_value := data.get("severityNumber")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "severity_number")
            _args["severity_number"] = SeverityNumber(_value)
        if (_value := data.get("severityText")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "severity_text")
            _args["severity_text"] = _value
        if (_value := data.get("body")) is not None:
            _args["body"] = opentelemetry.proto_json.common.v1.common.AnyValue.from_dict(_value)
        if (_value := data.get("attributes")) is not None:
            _args["attributes"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.common.v1.common.KeyValue.from_dict(_v), "attributes")
        if (_value := data.get("droppedAttributesCount")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "dropped_attributes_count")
            _args["dropped_attributes_count"] = _value
        if (_value := data.get("flags")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "flags")
            _args["flags"] = _value
        if (_value := data.get("traceId")) is not None:
            _args["trace_id"] = opentelemetry.proto_json._json_codec.decode_hex(_value, "trace_id")
        if (_value := data.get("spanId")) is not None:
            _args["span_id"] = opentelemetry.proto_json._json_codec.decode_hex(_value, "span_id")
        if (_value := data.get("eventName")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "event_name")
            _args["event_name"] = _value

        return cls(**_args)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from logging import getLogger

from opentelemetry.sdk.error_handler import ErrorHandler

logger = getLogger(__name__)


# pylint: disable=too-many-ancestors
class ErrorHandler1(ErrorHandler, IndexError, KeyError):
    def _handle(self, error: Exception, *args, **kwargs):
        if isinstance(error, IndexError):
            logger.exception("ErrorHandler1 handling an IndexError")

        elif isinstance(error, KeyError):
            logger.exception("ErrorHandler1 handling a KeyError")



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from enum import Enum
from typing import Final

GRAPHQL_DOCUMENT: Final = "graphql.document"
"""
The GraphQL document being executed.
Note: If instrumentation can reliably identify and redact sensitive information it SHOULD do it.
"""

GRAPHQL_OPERATION_NAME: Final = "graphql.operation.name"
"""
The name of the operation being executed.
"""

GRAPHQL_OPERATION_TYPE: Final = "graphql.operation.type"
"""
The type of the operation being executed.
"""


class GraphqlOperationTypeValues(Enum):
    QUERY = "query"
    """GraphQL query."""
    MUTATION = "mutation"
    """GraphQL mutation."""
    SUBSCRIPTION = "subscription"
    """GraphQL subscription."""



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# A default event name to be used for logging events when a better event name
# can't be derived from the event's key-value pairs.
DEFAULT_EVENT_NAME = "log"


def time_seconds_to_ns(time_seconds):
    """Converts a time value in seconds to a time value in nanoseconds.

    `time_seconds` is a `float` as returned by `time.time()` which represents
    the number of seconds since the epoch.

    The returned value is an `int` representing the number of nanoseconds since
    the epoch.
    """

    return int(time_seconds * 1e9)


def time_seconds_from_ns(time_nanoseconds):
    """Converts a time value in nanoseconds to a time value in seconds.

    `time_nanoseconds` is an `int` representing the number of nanoseconds since
    the epoch.

    The returned value is a `float` representing the number of seconds since
    the epoch.
    """

    return time_nanoseconds / 1e9


def event_name_from_kv(key_values):
    """A helper function which returns an event name from the given dict, or a
    default event name.
    """

    if key_values is None or "event" not in key_values:
        return DEFAULT_EVENT_NAME

    return key_values["event"]



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import io
import wsgiref.util as wsgiref_util

from opentelemetry import trace
from opentelemetry.test.test_base import TestBase


class WsgiTestBase(TestBase):
    def setUp(self):
        super().setUp()

        self.write_buffer = io.BytesIO()
        self.write = self.write_buffer.write

        self.environ = {}
        wsgiref_util.setup_testing_defaults(self.environ)

        self.status = None
        self.response_headers = None
        self.exc_info = None

    def start_response(self, status, response_headers, exc_info=None):
        self.status = status
        self.response_headers = response_headers
        self.exc_info = exc_info
        return self.write

    def assertTraceResponseHeaderMatchesSpan(self, headers, span):  # pylint: disable=invalid-name
        self.assertIn("traceresponse", headers)
        self.assertEqual(
            headers["access-control-expose-headers"],
            "traceresponse",
        )

        trace_id = trace.format_trace_id(span.get_span_context().trace_id)
        span_id = trace.format_span_id(span.get_span_context().span_id)
        trace_flags = span.get_span_context().trace_flags
        self.assertEqual(
            f"00-{trace_id}-{span_id}-{trace_flags:02x}",
            headers["traceresponse"],
        )



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import abc
import typing
from collections.abc import Iterable, Mapping, MutableMapping

from opentelemetry.context.context import Context

CarrierT = typing.TypeVar("CarrierT")
# pylint: disable=invalid-name
CarrierValT = list[str] | str


class Getter(abc.ABC, typing.Generic[CarrierT]):
    """This class implements a Getter that enables extracting propagated
    fields from a carrier.
    """

    @abc.abstractmethod
    def get(self, carrier: CarrierT, key: str) -> list[str] | None:
        """Function that can retrieve zero
        or more values from the carrier. In the case that
        the value does not exist, returns None.

        Args:
            carrier: An object which contains values that are used to
                    construct a Context.
            key: key of a field in carrier.
        Returns: first value of the propagation key or None if the key doesn't
                exist.
        """

    @abc.abstractmethod
    def keys(self, carrier: CarrierT) -> list[str]:
        """Function that can retrieve all the keys in a carrier object.

        Args:
            carrier: An object which contains values that are
                used to construct a Context.
        Returns:
            list of keys from the carrier.
        """


class Setter(abc.ABC, typing.Generic[CarrierT]):
    """This class implements a Setter that enables injecting propagated
    fields into a carrier.
    """

    @abc.abstractmethod
    def set(self, carrier: CarrierT, key: str, value: str) -> None:
        """Function that can set a value into a carrier""

        Args:
            carrier: An object which contains values that are used to
                    construct a Context.
            key: key of a field in carrier.
            value: value for a field in carrier.
        """


class DefaultGetter(Getter[Mapping[str, CarrierValT]]):
    def get(
        self, carrier: Mapping[str, CarrierValT], key: str
    ) -> list[str] | None:
        """Getter implementation to retrieve a value from a dictionary.

        Args:
            carrier: dictionary in which to get value
            key: the key used to get the value
        Returns:
            A list with a single string with the value if it exists, else None.
        """
        val = carrier.get(key, None)
        if val is None:
            return None
        if isinstance(val, Iterable) and not isinstance(val, str):
            return list(val)
        return [val]

    def keys(self, carrier: Mapping[str, CarrierValT]) -> list[str]:
        """Keys implementation that returns all keys from a dictionary."""
        return list(carrier.keys())


default_getter: Getter[CarrierT] = DefaultGetter()  # type: ignore


class DefaultSetter(Setter[MutableMapping[str, CarrierValT]]):
    def set(
        self,
        carrier: MutableMapping[str, CarrierValT],
        key: str,
        value: CarrierValT,
    ) -> None:
        """Setter implementation to set a value into a dictionary.

        Args:
            carrier: dictionary in which to set value
            key: the key used to set the value
            value: the value to set
        """
        carrier[key] = value


default_setter: Setter[CarrierT] = DefaultSetter()  # type: ignore


class TextMapPropagator(abc.ABC):
    """This class provides an interface that enables extracting and injecting
    context into headers of HTTP requests. HTTP frameworks and clients
    can integrate with TextMapPropagator by providing the object containing the
    headers, and a getter and setter function for the extraction and
    injection of values, respectively.

    """

    @abc.abstractmethod
    def extract(
        self,
        carrier: CarrierT,
        context: Context | None = None,
        getter: Getter[CarrierT] = default_getter,
    ) -> Context:
        """Create a Context from values in the carrier.

        The extract function should retrieve values from the carrier
        object using getter, and use values to populate a
        Context value and return it.

        Args:
            getter: a function that can retrieve zero
                or more values from the carrier. In the case that
                the value does not exist, return an empty list.
            carrier: and object which contains values that are
                used to construct a Context. This object
                must be paired with an appropriate getter
                which understands how to extract a value from it.
            context: an optional Context to use. Defaults to root
                context if not set.
        Returns:
            A Context with configuration found in the carrier.

        """

    @abc.abstractmethod
    def inject(
        self,
        carrier: CarrierT,
        context: Context | None = None,
        setter: Setter[CarrierT] = default_setter,
    ) -> None:
        """Inject values from a Context into a carrier.

        inject enables the propagation of values into HTTP clients or
        other objects which perform an HTTP request. Implementations
        should use the `Setter` 's set method to set values on the
        carrier.

        Args:
            carrier: An object that a place to define HTTP headers.
                Should be paired with setter, which should
                know how to set header values on the carrier.
            context: an optional Context to use. Defaults to current
                context if not set.
            setter: An optional `Setter` object that can set values
                on the carrier.

        """

    @property
    @abc.abstractmethod
    def fields(self) -> set[str]:
        """
        Gets the fields set in the carrier by the `inject` method.

        If the carrier is reused, its fields that correspond with the ones
        present in this attribute should be deleted before calling `inject`.

        Returns:
            A set with the fields set in `inject`.
        """



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

import os
import weakref
from atexit import register, unregister
from collections.abc import Callable, Sequence
from dataclasses import dataclass
from logging import getLogger
from os import environ
from threading import Lock
from time import time_ns

# This kind of import is needed to avoid Sphinx errors.
import opentelemetry.sdk.metrics
from opentelemetry.metrics import Counter as APICounter
from opentelemetry.metrics import Histogram as APIHistogram
from opentelemetry.metrics import Meter as APIMeter
from opentelemetry.metrics import MeterProvider as APIMeterProvider
from opentelemetry.metrics import NoOpMeter
from opentelemetry.metrics import ObservableCounter as APIObservableCounter
from opentelemetry.metrics import ObservableGauge as APIObservableGauge
from opentelemetry.metrics import (
    ObservableUpDownCounter as APIObservableUpDownCounter,
)
from opentelemetry.metrics import UpDownCounter as APIUpDownCounter
from opentelemetry.metrics import _Gauge as APIGauge
from opentelemetry.sdk.environment_variables import (
    OTEL_METRICS_EXEMPLAR_FILTER,
    OTEL_SDK_DISABLED,
)
from opentelemetry.sdk.metrics._internal.exceptions import MetricsTimeoutError
from opentelemetry.sdk.metrics._internal.exemplar import (
    AlwaysOffExemplarFilter,
    AlwaysOnExemplarFilter,
    ExemplarFilter,
    TraceBasedExemplarFilter,
)
from opentelemetry.sdk.metrics._internal.instrument import (
    _Counter,
    _Gauge,
    _Histogram,
    _ObservableCounter,
    _ObservableGauge,
    _ObservableUpDownCounter,
    _UpDownCounter,
)
from opentelemetry.sdk.metrics._internal.measurement_consumer import (
    MeasurementConsumer,
    SynchronousMeasurementConsumer,
)
from opentelemetry.sdk.metrics._internal.sdk_configuration import (
    SdkConfiguration,
)
from opentelemetry.sdk.resources import (
    Resource,
    _get_process_dependent_resource,
)
from opentelemetry.sdk.util._configurator import RuleBasedConfigurator
from opentelemetry.sdk.util.instrumentation import (
    InstrumentationScope,
)
from opentelemetry.util._once import Once
from opentelemetry.util.types import (
    Attributes,
)

_logger = getLogger(__name__)


@dataclass
class _MeterConfig:
    is_enabled: bool = True

    @classmethod
    def default(cls) -> "_MeterConfig":
        return _MeterConfig()


class _ProxyMeterConfig:
    def __init__(self, config: _MeterConfig):
        self._config = config

    @property
    def is_enabled(self) -> bool:
        return self._config.is_enabled

    def update(self, config: _MeterConfig) -> None:
        self._config = config


class Meter(APIMeter):
    """See `opentelemetry.metrics.Meter`."""

    def __init__(
        self,
        instrumentation_scope: InstrumentationScope,
        measurement_consumer: MeasurementConsumer,
        *,
        _meter_config: _MeterConfig | None = None,
    ):
        super().__init__(
            name=instrumentation_scope.name,
            version=instrumentation_scope.version,
            schema_url=instrumentation_scope.schema_url,
        )
        self._instrumentation_scope = instrumentation_scope
        self._measurement_consumer = measurement_consumer
        self._instrument_id_instrument = {}
        self._instrument_registration_lock = Lock()
        self._meter_config = _ProxyMeterConfig(
            _meter_config or _MeterConfig.default()
        )

    def _is_enabled(self) -> bool:
        return self._meter_config.is_enabled

    def _set_meter_config(self, meter_config: _MeterConfig) -> None:
        self._meter_config.update(meter_config)

    def create_counter(self, name, unit="", description="") -> APICounter:
        with self._instrument_registration_lock:
            status = self._register_instrument(
                name, _Counter, unit, description
            )
            if not status.already_registered:
                self._instrument_id_instrument[status.instrument_id] = (
                    _Counter(
                        name,
                        self._instrumentation_scope,
                        self._measurement_consumer,
                        unit,
                        description,
                        _meter_config=self._meter_config,
                    )
                )
            instrument = self._instrument_id_instrument[status.instrument_id]

        if status.conflict:
            # FIXME #2558 go through all views here and check if this
            # instrument registration conflict can be fixed. If it can be, do
            # not log the following warning.
            self._log_instrument_registration_conflict(
                name,
                APICounter.__name__,
                unit,
                description,
                status,
            )
        return instrument

    def create_up_down_counter(
        self, name, unit="", description=""
    ) -> APIUpDownCounter:
        with self._instrument_registration_lock:
            status = self._register_instrument(
                name, _UpDownCounter, unit, description
            )
            if not status.already_registered:
                self._instrument_id_instrument[status.instrument_id] = (
                    _UpDownCounter(
                        name,
                        self._instrumentation_scope,
                        self._measurement_consumer,
                        unit,
                        description,
                        _meter_config=self._meter_config,
                    )
                )
            instrument = self._instrument_id_instrument[status.instrument_id]

        if status.conflict:
            # FIXME #2558 go through all views here and check if this
            # instrument registration conflict can be fixed. If it can be, do
            # not log the following warning.
            self._log_instrument_registration_conflict(
                name,
                APIUpDownCounter.__name__,
                unit,
                description,
                status,
            )
        return instrument

    def create_observable_counter(
        self,
        name,
        callbacks=None,
        unit="",
        description="",
    ) -> APIObservableCounter:
        with self._instrument_registration_lock:
            status = self._register_instrument(
                name, _ObservableCounter, unit, description
            )
            if not status.already_registered:
                self._instrument_id_instrument[status.instrument_id] = (
                    _ObservableCounter(
                        name,
                        self._instrumentation_scope,
                        self._measurement_consumer,
                        callbacks,
                        unit,
                        description,
                        _meter_config=self._meter_config,
                    )
                )
            instrument = self._instrument_id_instrument[status.instrument_id]

        if not status.already_registered:
            self._measurement_consumer.register_asynchronous_instrument(
                instrument
            )

        if status.conflict:
            # FIXME #2558 go through all views here and check if this
            # instrument registration conflict can be fixed. If it can be, do
            # not log the following warning.
            self._log_instrument_registration_conflict(
                name,
                APIObservableCounter.__name__,
                unit,
                description,
                status,
            )
        return instrument

    def create_histogram(
        self,
        name: str,
        unit: str = "",
        description: str = "",
        *,
        explicit_bucket_boundaries_advisory: Sequence[float] | None = None,
    ) -> APIHistogram:
        if explicit_bucket_boundaries_advisory is not None:
            invalid_advisory = False
            if isinstance(explicit_bucket_boundaries_advisory, Sequence):
                try:
                    invalid_advisory = not (
                        all(
                            isinstance(e, (float, int))
                            for e in explicit_bucket_boundaries_advisory
                        )
                    )
                except (KeyError, TypeError):
                    invalid_advisory = True
            else:
                invalid_advisory = True

            if invalid_advisory:
                explicit_bucket_boundaries_advisory = None
                _logger.warning(
                    "explicit_bucket_boundaries_advisory must be a sequence of numbers"
                )

        with self._instrument_registration_lock:
            status = self._register_instrument(
                name,
                _Histogram,
                unit,
                description,
                explicit_bucket_boundaries_advisory,
            )
            if not status.already_registered:
                self._instrument_id_instrument[status.instrument_id] = (
                    _Histogram(
                        name,
                        self._instrumentation_scope,
                        self._measurement_consumer,
                        unit,
                        description,
                        explicit_bucket_boundaries_advisory,
                        _meter_config=self._meter_config,
                    )
                )
            instrument = self._instrument_id_instrument[status.instrument_id]

        if status.conflict:
            # FIXME #2558 go through all views here and check if this
            # instrument registration conflict can be fixed. If it can be, do
            # not log the following warning.
            self._log_instrument_registration_conflict(
                name,
                APIHistogram.__name__,
                unit,
                description,
                status,
            )
        return instrument

    def create_gauge(self, name, unit="", description="") -> APIGauge:
        with self._instrument_registration_lock:
            status = self._register_instrument(name, _Gauge, unit, description)
            if not status.already_registered:
                self._instrument_id_instrument[status.instrument_id] = _Gauge(
                    name,
                    self._instrumentation_scope,
                    self._measurement_consumer,
                    unit,
                    description,
                    _meter_config=self._meter_config,
                )
            instrument = self._instrument_id_instrument[status.instrument_id]

        if status.conflict:
            # FIXME #2558 go through all views here and check if this
            # instrument registration conflict can be fixed. If it can be, do
            # not log the following warning.
            self._log_instrument_registration_conflict(
                name,
                APIGauge.__name__,
                unit,
                description,
                status,
            )
        return instrument

    def create_observable_gauge(
        self, name, callbacks=None, unit="", description=""
    ) -> APIObservableGauge:
        with self._instrument_registration_lock:
            status = self._register_instrument(
                name, _ObservableGauge, unit, description
            )
            if not status.already_registered:
                self._instrument_id_instrument[status.instrument_id] = (
                    _ObservableGauge(
                        name,
                        self._instrumentation_scope,
                        self._measurement_consumer,
                        callbacks,
                        unit,
                        description,
                        _meter_config=self._meter_config,
                    )
                )
            instrument = self._instrument_id_instrument[status.instrument_id]

        if not status.already_registered:
            self._measurement_consumer.register_asynchronous_instrument(
                instrument
            )

        if status.conflict:
            # FIXME #2558 go through all views here and check if this
            # instrument registration conflict can be fixed. If it can be, do
            # not log the following warning.
            self._log_instrument_registration_conflict(
                name,
                APIObservableGauge.__name__,
                unit,
                description,
                status,
            )
        return instrument

    def create_observable_up_down_counter(
        self, name, callbacks=None, unit="", description=""
    ) -> APIObservableUpDownCounter:
        with self._instrument_registration_lock:
            status = self._register_instrument(
                name, _ObservableUpDownCounter, unit, description
            )
            if not status.already_registered:
                self._instrument_id_instrument[status.instrument_id] = (
                    _ObservableUpDownCounter(
                        name,
                        self._instrumentation_scope,
                        self._measurement_consumer,
                        callbacks,
                        unit,
                        description,
                        _meter_config=self._meter_config,
                    )
                )
            instrument = self._instrument_id_instrument[status.instrument_id]

        if not status.already_registered:
            self._measurement_consumer.register_asynchronous_instrument(
                instrument
            )

        if status.conflict:
            # FIXME #2558 go through all views here and check if this
            # instrument registration conflict can be fixed. If it can be, do
            # not log the following warning.
            self._log_instrument_registration_conflict(
                name,
                APIObservableUpDownCounter.__name__,
                unit,
                description,
                status,
            )
        return instrument


def _get_exemplar_filter(exemplar_filter: str) -> ExemplarFilter:
    if exemplar_filter == "trace_based":
        return TraceBasedExemplarFilter()
    if exemplar_filter == "always_on":
        return AlwaysOnExemplarFilter()
    if exemplar_filter == "always_off":
        return AlwaysOffExemplarFilter()
    msg = f"Unknown exemplar filter '{exemplar_filter}'."
    raise ValueError(msg)


_MeterConfiguratorT = Callable[[InstrumentationScope], _MeterConfig]
_RuleBasedMeterConfigurator = RuleBasedConfigurator[_MeterConfig]


def _default_meter_configurator(
    _meter_scope: InstrumentationScope,
) -> _MeterConfig:
    return _MeterConfig.default()


def _disable_meter_configurator(
    _meter_scope: InstrumentationScope,
) -> _MeterConfig:
    return _MeterConfig(is_enabled=False)


class MeterProvider(APIMeterProvider):
    r"""See `opentelemetry.metrics.MeterProvider`.

    Args:
        metric_readers: Register metric readers to collect metrics from the SDK
            on demand. Each :class:`opentelemetry.sdk.metrics.export.MetricReader` is
            completely independent and will collect separate streams of
            metrics. For push-based export, use
            :class:`opentelemetry.sdk.metrics.export.PeriodicExportingMetricReader`.
        resource: The resource representing what the metrics emitted from the SDK pertain to.
        shutdown_on_exit: If true, registers an `atexit` handler to call
            `MeterProvider.shutdown`
        views: The views to configure the metric output the SDK

    .. code-block:: python
        :caption: Push-based export with PeriodicExportingMetricReader

        from opentelemetry.sdk.metrics import MeterProvider
        from opentelemetry.sdk.metrics.export import (
            ConsoleMetricExporter,
            PeriodicExportingMetricReader,
        )

        reader = PeriodicExportingMetricReader(ConsoleMetricExporter())
        provider = MeterProvider(metric_readers=[reader])

    By default, instruments which do not match any :class:`opentelemetry.sdk.metrics.view.View` (or if no :class:`opentelemetry.sdk.metrics.view.View`\ s
    are provided) will report metrics with the default aggregation for the
    instrument's kind. To disable instruments by default, configure a match-all
    :class:`opentelemetry.sdk.metrics.view.View` with `DropAggregation` and then create :class:`opentelemetry.sdk.metrics.view.View`\ s to re-enable
    individual instruments:

    .. code-block:: python
        :caption: Disable default views

        MeterProvider(
            views=[
                View(instrument_name="*", aggregation=DropAggregation()),
                View(instrument_name="mycounter"),
            ],
            # ...
        )
    """

    _all_metric_readers_lock = Lock()
    _all_metric_readers = weakref.WeakSet()

    def __init__(
        self,
        metric_readers: Sequence[
            "opentelemetry.sdk.metrics.export.MetricReader"
        ] = (),
        resource: Resource | None = None,
        exemplar_filter: ExemplarFilter | None = None,
        shutdown_on_exit: bool = True,
        views: Sequence["opentelemetry.sdk.metrics.view.View"] = (),
        *,
        _meter_configurator: _MeterConfiguratorT | None = None,
    ):
        self._lock = Lock()
        self._meter_lock = Lock()
        self._atexit_handler = None
        if resource is None:
            resource = Resource.create({})
        self._sdk_config = SdkConfiguration(
            exemplar_filter=(
                exemplar_filter
                or _get_exemplar_filter(
                    environ.get(OTEL_METRICS_EXEMPLAR_FILTER, "trace_based")
                )
            ),
            resource=resource,
            views=views,
        )
        self._metric_readers = metric_readers
        self._measurement_consumer = SynchronousMeasurementConsumer(
            sdk_config=self._sdk_config,
            metric_readers=metric_readers,
        )
        disabled = environ.get(OTEL_SDK_DISABLED, "")
        self._disabled = disabled.lower().strip() == "true"

        if shutdown_on_exit:
            self._atexit_handler = register(self.shutdown)

        self._meters: dict[InstrumentationScope, Meter] = {}
        self._shutdown_once = Once()
        self._shutdown = False
        self._meter_configurator = (
            _meter_configurator or _default_meter_configurator
        )

        for metric_reader in self._metric_readers:
            with self._all_metric_readers_lock:
                if metric_reader in self._all_metric_readers:
                    # pylint: disable=broad-exception-raised
                    raise Exception(
                        f"MetricReader {metric_reader} has been registered "
                        "already in other MeterProvider instance"
                    )

                self._all_metric_readers.add(metric_reader)

            metric_reader._set_collect_callback(
                self._measurement_consumer.collect
            )
            metric_reader._set_meter_provider(self)

        if hasattr(os, "register_at_fork"):
            weak_at_fork = weakref.WeakMethod(self._handle_fork)

            def _after_in_child() -> None:
                if at_fork := weak_at_fork():
                    at_fork()

            os.register_at_fork(after_in_child=_after_in_child)

    def _handle_fork(self) -> None:
        self._lock = Lock()
        self._meter_lock = Lock()
        type(self)._all_metric_readers_lock = Lock()
        self._update_resource(_get_process_dependent_resource())

    def _set_meter_configurator(
        self, *, meter_configurator: _MeterConfiguratorT
    ):
        """Set a new MeterConfigurator for this MeterProvider.

        Setting a new MeterConfigurator will result in the configurator being called
        for each outstanding Meter and for any newly created meters thereafter.
        Therefore, it is important that the provided function returns quickly.
        """
        with self._meter_lock:
            self._meter_configurator = meter_configurator
            for instrumentation_scope, meter in self._meters.items():
                # pylint: disable-next=protected-access
                meter._set_meter_config(
                    self._apply_meter_configurator(instrumentation_scope)
                )

    def _update_resource(self, resource: Resource) -> None:
        with self._meter_lock:
            self._sdk_config.resource = self._sdk_config.resource.merge(
                resource
            )

    def _apply_meter_configurator(
        self, instrumentation_scope: InstrumentationScope
    ) -> _MeterConfig:
        try:
            return self._meter_configurator(instrumentation_scope)
        # pylint: disable-next=broad-exception-caught
        except Exception:
            _logger.exception(
                "meter configurator failed for scope '%s', using default config",
                instrumentation_scope.name,
            )
            return _MeterConfig.default()

    def force_flush(self, timeout_millis: float = 10_000) -> bool:
        deadline_ns = time_ns() + timeout_millis * 10**6

        metric_reader_error = {}

        for metric_reader in self._metric_readers:
            current_ts = time_ns()
            try:
                if current_ts >= deadline_ns:
                    raise MetricsTimeoutError(
                        "Timed out while flushing metric readers"
                    )
                metric_reader.force_flush(
                    timeout_millis=(deadline_ns - current_ts) / 10**6
                )

            # pylint: disable=broad-exception-caught
            except Exception as error:
                metric_reader_error[metric_reader] = error

        if metric_reader_error:
            metric_reader_error_string = "\n".join(
                [
                    f"{metric_reader.__class__.__name__}: {repr(error)}"
                    for metric_reader, error in metric_reader_error.items()
                ]
            )

            # pylint: disable=broad-exception-raised
            raise Exception(
                "MeterProvider.force_flush failed because the following "
                "metric readers failed during collect:\n"
                f"{metric_reader_error_string}"
            )
        return True

    def shutdown(self, timeout_millis: float = 30_000):
        deadline_ns = time_ns() + timeout_millis * 10**6

        def _shutdown():
            self._shutdown = True

        did_shutdown = self._shutdown_once.do_once(_shutdown)

        if not did_shutdown:
            _logger.warning("shutdown can only be called once")
            return

        metric_reader_error = {}

        for metric_reader in self._metric_readers:
            current_ts = time_ns()
            try:
                if current_ts >= deadline_ns:
                    # pylint: disable=broad-exception-raised
                    raise Exception(
                        "Didn't get to execute, deadline already exceeded"
                    )
                metric_reader.shutdown(
                    timeout_millis=(deadline_ns - current_ts) / 10**6
                )

            # pylint: disable=broad-exception-caught
            except Exception as error:
                metric_reader_error[metric_reader] = error

        if self._atexit_handler is not None:
            unregister(self._atexit_handler)
            self._atexit_handler = None

        if metric_reader_error:
            metric_reader_error_string = "\n".join(
                [
                    f"{metric_reader.__class__.__name__}: {repr(error)}"
                    for metric_reader, error in metric_reader_error.items()
                ]
            )

            # pylint: disable=broad-exception-raised
            raise Exception(
                "MeterProvider.shutdown failed because the following "
                "metric readers failed during shutdown:\n"
                f"{metric_reader_error_string}"
            )

    def get_meter(
        self,
        name: str,
        version: str | None = None,
        schema_url: str | None = None,
        attributes: Attributes | None = None,
    ) -> APIMeter:
        if self._disabled:
            return NoOpMeter(name, version=version, schema_url=schema_url)

        if self._shutdown:
            _logger.warning(
                "A shutdown `MeterProvider` can not provide a `Meter`"
            )
            return NoOpMeter(name, version=version, schema_url=schema_url)

        if not name:
            _logger.warning("Meter name cannot be None or empty.")
            return NoOpMeter(name, version=version, schema_url=schema_url)

        instrumentation_scope = InstrumentationScope(
            name, version, schema_url, attributes
        )
        with self._meter_lock:
            if not self._meters.get(instrumentation_scope):
                # FIXME #2558 pass SDKConfig object to meter so that the meter
                # has access to views.
                self._meters[instrumentation_scope] = Meter(
                    instrumentation_scope,
                    self._measurement_consumer,
                    _meter_config=self._apply_meter_configurator(
                        instrumentation_scope
                    ),
                )
            return self._meters[instrumentation_scope]

    def add_metric_reader(
        self, metric_reader: "opentelemetry.sdk.metrics.export.MetricReader"
    ) -> None:
        with self._all_metric_readers_lock:
            if metric_reader in self._all_metric_readers:
                _logger.warning(
                    "MetricReader '%s' has been registered already!",
                    metric_reader,
                )
                return
            self._measurement_consumer.add_metric_reader(metric_reader)
            # pylint: disable-next=protected-access
            metric_reader._set_collect_callback(
                self._measurement_consumer.collect
            )
            self._all_metric_readers.add(metric_reader)

    def remove_metric_reader(
        self,
        metric_reader: "opentelemetry.sdk.metrics.export.MetricReader",
    ) -> None:
        with self._all_metric_readers_lock:
            if metric_reader not in self._all_metric_readers:
                _logger.warning(
                    "MetricReader '%s' has not been registered!", metric_reader
                )
                return
            self._measurement_consumer.remove_metric_reader(metric_reader)
            # pylint: disable-next=protected-access
            metric_reader._set_collect_callback(None)
            metric_reader.shutdown()
            self._all_metric_readers.remove(metric_reader)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# flask_example.py
import flask
import requests

from opentelemetry import trace
from opentelemetry.instrumentation.flask import FlaskInstrumentor
from opentelemetry.instrumentation.requests import RequestsInstrumentor
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import (
    BatchSpanProcessor,
    ConsoleSpanExporter,
)

trace.set_tracer_provider(TracerProvider())
trace.get_tracer_provider().add_span_processor(
    BatchSpanProcessor(ConsoleSpanExporter())
)

app = flask.Flask(__name__)
FlaskInstrumentor().instrument_app(app)
RequestsInstrumentor().instrument()

tracer = trace.get_tracer(__name__)


@app.route("/")
def hello():
    with tracer.start_as_current_span("example-request"):
        requests.get("http://www.example.com", timeout=10)
    return "hello"


app.run(port=5000)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# pylint:disable=protected-access,no-self-use,no-member

import logging
import threading
import time
import unittest
from abc import ABC, abstractmethod
from unittest.mock import Mock

from opentelemetry._logs import LogRecord, SeverityNumber
from opentelemetry.sdk._logs._internal import (
    ConcurrentMultiLogRecordProcessor,
    LoggerProvider,
    LoggingHandler,
    LogRecordProcessor,
    ReadWriteLogRecord,
    SynchronousMultiLogRecordProcessor,
)


class AnotherLogRecordProcessor(LogRecordProcessor):
    def __init__(self, exporter, logs_list):
        self._exporter = exporter
        self._log_list = logs_list
        self._closed = False

    def on_emit(self, log_record: ReadWriteLogRecord):
        if self._closed:
            return
        self._log_list.append(
            (
                log_record.log_record.body,
                log_record.log_record.severity_text,
            )
        )

    def shutdown(self):
        self._closed = True
        self._exporter.shutdown()

    def force_flush(self, timeout_millis=30000):
        self._log_list.clear()
        return True


class TestLogRecordProcessor(unittest.TestCase):
    def test_log_record_processor(self):
        provider = LoggerProvider()
        handler = LoggingHandler(logger_provider=provider)

        logs_list_1 = []
        processor1 = AnotherLogRecordProcessor(Mock(), logs_list_1)
        logs_list_2 = []
        processor2 = AnotherLogRecordProcessor(Mock(), logs_list_2)

        logger = logging.getLogger("test.span.processor")
        logger.addHandler(handler)

        # Test no proessor added
        with self.assertLogs(level=logging.CRITICAL):
            logger.critical("Odisha, we have another major cyclone")

        self.assertEqual(len(logs_list_1), 0)
        self.assertEqual(len(logs_list_2), 0)

        # Add one processor
        provider.add_log_record_processor(processor1)
        with self.assertLogs(level=logging.WARNING):
            logger.warning("Brace yourself")
        with self.assertLogs(level=logging.ERROR):
            logger.error("Some error message")

        expected_list_1 = [
            ("Brace yourself", "WARN"),
            ("Some error message", "ERROR"),
        ]
        self.assertEqual(logs_list_1, expected_list_1)

        # Add another processor
        provider.add_log_record_processor(processor2)
        with self.assertLogs(level=logging.CRITICAL):
            logger.critical("Something disastrous")
        expected_list_1.append(("Something disastrous", "FATAL"))

        expected_list_2 = [("Something disastrous", "FATAL")]

        self.assertEqual(logs_list_1, expected_list_1)
        self.assertEqual(logs_list_2, expected_list_2)


class MultiLogRecordProcessorTestBase(ABC):
    @abstractmethod
    def _get_multi_log_record_processor(self):
        pass

    def make_record(self):
        return ReadWriteLogRecord(
            LogRecord(
                timestamp=1622300111608942000,
                severity_text="WARN",
                severity_number=SeverityNumber.WARN,
                body="Warning message",
            )
        )

    def test_on_emit(self):
        multi_log_record_processor = self._get_multi_log_record_processor()
        mocks = [Mock(spec=LogRecordProcessor) for _ in range(5)]
        for mock in mocks:
            multi_log_record_processor.add_log_record_processor(mock)
        record = self.make_record()
        multi_log_record_processor.on_emit(record)
        for mock in mocks:
            mock.on_emit.assert_called_with(record)
        multi_log_record_processor.shutdown()

    def test_on_shutdown(self):
        multi_log_record_processor = self._get_multi_log_record_processor()
        mocks = [Mock(spec=LogRecordProcessor) for _ in range(5)]
        for mock in mocks:
            multi_log_record_processor.add_log_record_processor(mock)
        multi_log_record_processor.shutdown()
        for mock in mocks:
            mock.shutdown.assert_called_once_with()

    def test_on_force_flush(self):
        multi_log_record_processor = self._get_multi_log_record_processor()
        mocks = [Mock(spec=LogRecordProcessor) for _ in range(5)]
        for mock in mocks:
            multi_log_record_processor.add_log_record_processor(mock)
        ret_value = multi_log_record_processor.force_flush(100)

        self.assertTrue(ret_value)
        for mock_processor in mocks:
            self.assertEqual(1, mock_processor.force_flush.call_count)


class TestSynchronousMultiLogRecordProcessor(
    MultiLogRecordProcessorTestBase, unittest.TestCase
):
    def _get_multi_log_record_processor(self):
        return SynchronousMultiLogRecordProcessor()

    def test_force_flush_delayed(self):
        multi_log_record_processor = SynchronousMultiLogRecordProcessor()

        def delay(_):
            time.sleep(0.09)

        mock_processor1 = Mock(spec=LogRecordProcessor)
        mock_processor1.force_flush = Mock(side_effect=delay)
        multi_log_record_processor.add_log_record_processor(mock_processor1)
        mock_processor2 = Mock(spec=LogRecordProcessor)
        multi_log_record_processor.add_log_record_processor(mock_processor2)

        ret_value = multi_log_record_processor.force_flush(50)
        self.assertFalse(ret_value)
        self.assertEqual(mock_processor1.force_flush.call_count, 1)
        self.assertEqual(mock_processor2.force_flush.call_count, 0)

    def test_force_flush_processor_returns_false(self):
        multi_log_record_processor = SynchronousMultiLogRecordProcessor()

        mock_processor1 = Mock(spec=LogRecordProcessor)
        mock_processor1.force_flush = Mock(return_value=False)
        multi_log_record_processor.add_log_record_processor(mock_processor1)
        mock_processor2 = Mock(spec=LogRecordProcessor)
        mock_processor2.force_flush = Mock(return_value=True)
        multi_log_record_processor.add_log_record_processor(mock_processor2)

        ret_value = multi_log_record_processor.force_flush(50)
        self.assertFalse(ret_value)
        self.assertEqual(mock_processor1.force_flush.call_count, 1)
        self.assertEqual(mock_processor2.force_flush.call_count, 1)

    def test_force_flush_processor_returns_none(self):
        multi_log_record_processor = SynchronousMultiLogRecordProcessor()

        mock_processor1 = Mock(spec=LogRecordProcessor)
        mock_processor1.force_flush = Mock(return_value=None)
        multi_log_record_processor.add_log_record_processor(mock_processor1)
        mock_processor2 = Mock(spec=LogRecordProcessor)
        mock_processor2.force_flush = Mock(return_value=True)
        multi_log_record_processor.add_log_record_processor(mock_processor2)

        ret_value = multi_log_record_processor.force_flush(50)
        self.assertTrue(ret_value)
        self.assertEqual(mock_processor1.force_flush.call_count, 1)
        self.assertEqual(mock_processor2.force_flush.call_count, 1)


class TestConcurrentMultiLogRecordProcessor(
    MultiLogRecordProcessorTestBase, unittest.TestCase
):
    def _get_multi_log_record_processor(self):
        return ConcurrentMultiLogRecordProcessor()

    def test_force_flush_delayed(self):
        multi_log_record_processor = ConcurrentMultiLogRecordProcessor()
        wait_event = threading.Event()

        def delay(_):
            wait_event.wait()

        mock1 = Mock(spec=LogRecordProcessor)
        mock1.force_flush = Mock(side_effect=delay)
        mocks = [Mock(LogRecordProcessor) for _ in range(5)]
        mocks = [mock1] + mocks
        for mock_processor in mocks:
            multi_log_record_processor.add_log_record_processor(mock_processor)

        ret_value = multi_log_record_processor.force_flush(50)
        wait_event.set()

        self.assertFalse(ret_value)
        for mock in mocks:
            self.assertEqual(1, mock.force_flush.call_count)
        multi_log_record_processor.shutdown()

    def test_force_flush_processor_returns_false(self):
        multi_log_record_processor = ConcurrentMultiLogRecordProcessor()

        false_mock = Mock(spec=LogRecordProcessor)
        false_mock.force_flush = Mock(return_value=False)
        mocks = [Mock(spec=LogRecordProcessor) for _ in range(4)]
        mocks.insert(0, false_mock)

        for mock_processor in mocks:
            multi_log_record_processor.add_log_record_processor(mock_processor)

        ret_value = multi_log_record_processor.force_flush()

        self.assertFalse(ret_value)
        for mock in mocks:
            self.assertEqual(1, mock.force_flush.call_count)
        multi_log_record_processor.shutdown()

    def test_force_flush_processor_returns_none(self):
        multi_log_record_processor = ConcurrentMultiLogRecordProcessor()

        none_mock = Mock(spec=LogRecordProcessor)
        none_mock.force_flush = Mock(return_value=None)
        mocks = [Mock(spec=LogRecordProcessor) for _ in range(4)]
        mocks.insert(0, none_mock)

        for mock_processor in mocks:
            multi_log_record_processor.add_log_record_processor(mock_processor)

        ret_value = multi_log_record_processor.force_flush()

        self.assertTrue(ret_value)
        for mock in mocks:
            self.assertEqual(1, mock.force_flush.call_count)
        multi_log_record_processor.shutdown()



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

"""
This example shows how to export metrics to multiple destinations.
Each PeriodicExportingMetricReader has its own collection interval
and export queue, so destinations do not block each other.
"""

from opentelemetry import metrics
from opentelemetry.exporter.otlp.proto.grpc.metric_exporter import (
    OTLPMetricExporter as GrpcMetricExporter,
)
from opentelemetry.exporter.otlp.proto.http.metric_exporter import (
    OTLPMetricExporter as HttpMetricExporter,
)
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import (
    ConsoleMetricExporter,
    PeriodicExportingMetricReader,
)

# Destination 1: OTLP over gRPC
grpc_reader = PeriodicExportingMetricReader(
    GrpcMetricExporter(endpoint="http://localhost:4317", insecure=True)
)

# Destination 2: OTLP over HTTP
http_reader = PeriodicExportingMetricReader(
    HttpMetricExporter(endpoint="http://localhost:4318/v1/metrics")
)

# Destination 3: Console (for debugging)
console_reader = PeriodicExportingMetricReader(ConsoleMetricExporter())

# Pass all readers to the MeterProvider
provider = MeterProvider(
    metric_readers=[grpc_reader, http_reader, console_reader]
)
metrics.set_meter_provider(provider)

meter = metrics.get_meter(__name__)
counter = meter.create_counter(
    "request.count", description="Number of requests"
)

counter.add(1, {"endpoint": "/api/users"})
counter.add(1, {"endpoint": "/api/orders"})

print("Metrics are exported to all three destinations.")

provider.shutdown()



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# AUTO-GENERATED from "opentelemetry/proto/metrics/v1/metrics.proto"
# DO NOT EDIT MANUALLY

from __future__ import annotations

import builtins
import dataclasses
import enum
import functools
import typing

_dataclass = functools.partial(dataclasses.dataclass, slots=True)

import opentelemetry.proto_json._json_codec
import opentelemetry.proto_json.common.v1.common
import opentelemetry.proto_json.resource.v1.resource


@typing.final
class AggregationTemporality(enum.IntEnum):
    """
    Generated from protobuf enum AggregationTemporality
    """

    AGGREGATION_TEMPORALITY_UNSPECIFIED = 0
    AGGREGATION_TEMPORALITY_DELTA = 1
    AGGREGATION_TEMPORALITY_CUMULATIVE = 2

@typing.final
class DataPointFlags(enum.IntEnum):
    """
    Generated from protobuf enum DataPointFlags
    """

    DATA_POINT_FLAGS_DO_NOT_USE = 0
    DATA_POINT_FLAGS_NO_RECORDED_VALUE_MASK = 1

@typing.final
@_dataclass
class MetricsData(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message MetricsData
    """

    resource_metrics: builtins.list[ResourceMetrics] = dataclasses.field(default_factory=builtins.list)

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.resource_metrics:
            _result["resourceMetrics"] = opentelemetry.proto_json._json_codec.encode_repeated(self.resource_metrics, lambda _v: _v.to_dict())
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "MetricsData":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            MetricsData instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("resourceMetrics")) is not None:
            _args["resource_metrics"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: ResourceMetrics.from_dict(_v), "resource_metrics")

        return cls(**_args)


@typing.final
@_dataclass
class ResourceMetrics(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ResourceMetrics
    """

    resource: opentelemetry.proto_json.resource.v1.resource.Resource | None = None
    scope_metrics: builtins.list[ScopeMetrics] = dataclasses.field(default_factory=builtins.list)
    schema_url: builtins.str | None = ""

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.resource:
            _result["resource"] = self.resource.to_dict()
        if self.scope_metrics:
            _result["scopeMetrics"] = opentelemetry.proto_json._json_codec.encode_repeated(self.scope_metrics, lambda _v: _v.to_dict())
        if self.schema_url:
            _result["schemaUrl"] = self.schema_url
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ResourceMetrics":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ResourceMetrics instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("resource")) is not None:
            _args["resource"] = opentelemetry.proto_json.resource.v1.resource.Resource.from_dict(_value)
        if (_value := data.get("scopeMetrics")) is not None:
            _args["scope_metrics"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: ScopeMetrics.from_dict(_v), "scope_metrics")
        if (_value := data.get("schemaUrl")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "schema_url")
            _args["schema_url"] = _value

        return cls(**_args)


@typing.final
@_dataclass
class ScopeMetrics(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ScopeMetrics
    """

    scope: opentelemetry.proto_json.common.v1.common.InstrumentationScope | None = None
    metrics: builtins.list[Metric] = dataclasses.field(default_factory=builtins.list)
    schema_url: builtins.str | None = ""

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.scope:
            _result["scope"] = self.scope.to_dict()
        if self.metrics:
            _result["metrics"] = opentelemetry.proto_json._json_codec.encode_repeated(self.metrics, lambda _v: _v.to_dict())
        if self.schema_url:
            _result["schemaUrl"] = self.schema_url
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ScopeMetrics":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ScopeMetrics instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("scope")) is not None:
            _args["scope"] = opentelemetry.proto_json.common.v1.common.InstrumentationScope.from_dict(_value)
        if (_value := data.get("metrics")) is not None:
            _args["metrics"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: Metric.from_dict(_v), "metrics")
        if (_value := data.get("schemaUrl")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "schema_url")
            _args["schema_url"] = _value

        return cls(**_args)


@typing.final
@_dataclass
class Metric(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message Metric
    """

    name: builtins.str | None = ""
    description: builtins.str | None = ""
    unit: builtins.str | None = ""
    gauge: Gauge | None = None
    sum: Sum | None = None
    histogram: Histogram | None = None
    exponential_histogram: ExponentialHistogram | None = None
    summary: Summary | None = None
    metadata: builtins.list[opentelemetry.proto_json.common.v1.common.KeyValue] = dataclasses.field(default_factory=builtins.list)

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.name:
            _result["name"] = self.name
        if self.description:
            _result["description"] = self.description
        if self.unit:
            _result["unit"] = self.unit
        if self.metadata:
            _result["metadata"] = opentelemetry.proto_json._json_codec.encode_repeated(self.metadata, lambda _v: _v.to_dict())
        if self.summary is not None:
            _result["summary"] = self.summary.to_dict()
        elif self.exponential_histogram is not None:
            _result["exponentialHistogram"] = self.exponential_histogram.to_dict()
        elif self.histogram is not None:
            _result["histogram"] = self.histogram.to_dict()
        elif self.sum is not None:
            _result["sum"] = self.sum.to_dict()
        elif self.gauge is not None:
            _result["gauge"] = self.gauge.to_dict()
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "Metric":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            Metric instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("name")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "name")
            _args["name"] = _value
        if (_value := data.get("description")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "description")
            _args["description"] = _value
        if (_value := data.get("unit")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "unit")
            _args["unit"] = _value
        if (_value := data.get("metadata")) is not None:
            _args["metadata"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.common.v1.common.KeyValue.from_dict(_v), "metadata")
        if (_value := data.get("summary")) is not None:
            _args["summary"] = Summary.from_dict(_value)
        elif (_value := data.get("exponentialHistogram")) is not None:
            _args["exponential_histogram"] = ExponentialHistogram.from_dict(_value)
        elif (_value := data.get("histogram")) is not None:
            _args["histogram"] = Histogram.from_dict(_value)
        elif (_value := data.get("sum")) is not None:
            _args["sum"] = Sum.from_dict(_value)
        elif (_value := data.get("gauge")) is not None:
            _args["gauge"] = Gauge.from_dict(_value)

        return cls(**_args)


@typing.final
@_dataclass
class Gauge(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message Gauge
    """

    data_points: builtins.list[NumberDataPoint] = dataclasses.field(default_factory=builtins.list)

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.data_points:
            _result["dataPoints"] = opentelemetry.proto_json._json_codec.encode_repeated(self.data_points, lambda _v: _v.to_dict())
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "Gauge":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            Gauge instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("dataPoints")) is not None:
            _args["data_points"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: NumberDataPoint.from_dict(_v), "data_points")

        return cls(**_args)


@typing.final
@_dataclass
class Sum(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message Sum
    """

    data_points: builtins.list[NumberDataPoint] = dataclasses.field(default_factory=builtins.list)
    aggregation_temporality: AggregationTemporality | builtins.int | None = 0
    is_monotonic: builtins.bool | None = False

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.data_points:
            _result["dataPoints"] = opentelemetry.proto_json._json_codec.encode_repeated(self.data_points, lambda _v: _v.to_dict())
        if self.aggregation_temporality:
            _result["aggregationTemporality"] = builtins.int(self.aggregation_temporality)
        if self.is_monotonic:
            _result["isMonotonic"] = self.is_monotonic
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "Sum":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            Sum instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("dataPoints")) is not None:
            _args["data_points"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: NumberDataPoint.from_dict(_v), "data_points")
        if (_value := data.get("aggregationTemporality")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "aggregation_temporality")
            _args["aggregation_temporality"] = AggregationTemporality(_value)
        if (_value := data.get("isMonotonic")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.bool, "is_monotonic")
            _args["is_monotonic"] = _value

        return cls(**_args)


@typing.final
@_dataclass
class Histogram(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message Histogram
    """

    data_points: builtins.list[HistogramDataPoint] = dataclasses.field(default_factory=builtins.list)
    aggregation_temporality: AggregationTemporality | builtins.int | None = 0

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.data_points:
            _result["dataPoints"] = opentelemetry.proto_json._json_codec.encode_repeated(self.data_points, lambda _v: _v.to_dict())
        if self.aggregation_temporality:
            _result["aggregationTemporality"] = builtins.int(self.aggregation_temporality)
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "Histogram":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            Histogram instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("dataPoints")) is not None:
            _args["data_points"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: HistogramDataPoint.from_dict(_v), "data_points")
        if (_value := data.get("aggregationTemporality")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "aggregation_temporality")
            _args["aggregation_temporality"] = AggregationTemporality(_value)

        return cls(**_args)


@typing.final
@_dataclass
class ExponentialHistogram(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ExponentialHistogram
    """

    data_points: builtins.list[ExponentialHistogramDataPoint] = dataclasses.field(default_factory=builtins.list)
    aggregation_temporality: AggregationTemporality | builtins.int | None = 0

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.data_points:
            _result["dataPoints"] = opentelemetry.proto_json._json_codec.encode_repeated(self.data_points, lambda _v: _v.to_dict())
        if self.aggregation_temporality:
            _result["aggregationTemporality"] = builtins.int(self.aggregation_temporality)
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ExponentialHistogram":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ExponentialHistogram instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("dataPoints")) is not None:
            _args["data_points"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: ExponentialHistogramDataPoint.from_dict(_v), "data_points")
        if (_value := data.get("aggregationTemporality")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "aggregation_temporality")
            _args["aggregation_temporality"] = AggregationTemporality(_value)

        return cls(**_args)


@typing.final
@_dataclass
class Summary(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message Summary
    """

    data_points: builtins.list[SummaryDataPoint] = dataclasses.field(default_factory=builtins.list)

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.data_points:
            _result["dataPoints"] = opentelemetry.proto_json._json_codec.encode_repeated(self.data_points, lambda _v: _v.to_dict())
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "Summary":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            Summary instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("dataPoints")) is not None:
            _args["data_points"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: SummaryDataPoint.from_dict(_v), "data_points")

        return cls(**_args)


@typing.final
@_dataclass
class NumberDataPoint(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message NumberDataPoint
    """

    attributes: builtins.list[opentelemetry.proto_json.common.v1.common.KeyValue] = dataclasses.field(default_factory=builtins.list)
    start_time_unix_nano: builtins.int | None = 0
    time_unix_nano: builtins.int | None = 0
    as_double: builtins.float | None = None
    as_int: builtins.int | None = None
    exemplars: builtins.list[Exemplar] = dataclasses.field(default_factory=builtins.list)
    flags: builtins.int | None = 0

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.attributes:
            _result["attributes"] = opentelemetry.proto_json._json_codec.encode_repeated(self.attributes, lambda _v: _v.to_dict())
        if self.start_time_unix_nano:
            _result["startTimeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.start_time_unix_nano)
        if self.time_unix_nano:
            _result["timeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.time_unix_nano)
        if self.exemplars:
            _result["exemplars"] = opentelemetry.proto_json._json_codec.encode_repeated(self.exemplars, lambda _v: _v.to_dict())
        if self.flags:
            _result["flags"] = self.flags
        if self.as_int is not None:
            _result["asInt"] = opentelemetry.proto_json._json_codec.encode_int64(self.as_int)
        elif self.as_double is not None:
            _result["asDouble"] = opentelemetry.proto_json._json_codec.encode_float(self.as_double)
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "NumberDataPoint":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            NumberDataPoint instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("attributes")) is not None:
            _args["attributes"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.common.v1.common.KeyValue.from_dict(_v), "attributes")
        if (_value := data.get("startTimeUnixNano")) is not None:
            _args["start_time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "start_time_unix_nano")
        if (_value := data.get("timeUnixNano")) is not None:
            _args["time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "time_unix_nano")
        if (_value := data.get("exemplars")) is not None:
            _args["exemplars"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: Exemplar.from_dict(_v), "exemplars")
        if (_value := data.get("flags")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "flags")
            _args["flags"] = _value
        if (_value := data.get("asInt")) is not None:
            _args["as_int"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "as_int")
        elif (_value := data.get("asDouble")) is not None:
            _args["as_double"] = opentelemetry.proto_json._json_codec.decode_float(_value, "as_double")

        return cls(**_args)


@typing.final
@_dataclass
class HistogramDataPoint(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message HistogramDataPoint
    """

    attributes: builtins.list[opentelemetry.proto_json.common.v1.common.KeyValue] = dataclasses.field(default_factory=builtins.list)
    start_time_unix_nano: builtins.int | None = 0
    time_unix_nano: builtins.int | None = 0
    count: builtins.int | None = 0
    sum: builtins.float | None = None
    bucket_counts: builtins.list[builtins.int] = dataclasses.field(default_factory=builtins.list)
    explicit_bounds: builtins.list[builtins.float] = dataclasses.field(default_factory=builtins.list)
    exemplars: builtins.list[Exemplar] = dataclasses.field(default_factory=builtins.list)
    flags: builtins.int | None = 0
    min: builtins.float | None = None
    max: builtins.float | None = None

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.attributes:
            _result["attributes"] = opentelemetry.proto_json._json_codec.encode_repeated(self.attributes, lambda _v: _v.to_dict())
        if self.start_time_unix_nano:
            _result["startTimeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.start_time_unix_nano)
        if self.time_unix_nano:
            _result["timeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.time_unix_nano)
        if self.count:
            _result["count"] = opentelemetry.proto_json._json_codec.encode_int64(self.count)
        if self.sum is not None:
            _result["sum"] = opentelemetry.proto_json._json_codec.encode_float(self.sum)
        if self.bucket_counts:
            _result["bucketCounts"] = opentelemetry.proto_json._json_codec.encode_repeated(self.bucket_counts, lambda _v: opentelemetry.proto_json._json_codec.encode_int64(_v))
        if self.explicit_bounds:
            _result["explicitBounds"] = opentelemetry.proto_json._json_codec.encode_repeated(self.explicit_bounds, lambda _v: opentelemetry.proto_json._json_codec.encode_float(_v))
        if self.exemplars:
            _result["exemplars"] = opentelemetry.proto_json._json_codec.encode_repeated(self.exemplars, lambda _v: _v.to_dict())
        if self.flags:
            _result["flags"] = self.flags
        if self.min is not None:
            _result["min"] = opentelemetry.proto_json._json_codec.encode_float(self.min)
        if self.max is not None:
            _result["max"] = opentelemetry.proto_json._json_codec.encode_float(self.max)
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "HistogramDataPoint":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            HistogramDataPoint instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("attributes")) is not None:
            _args["attributes"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.common.v1.common.KeyValue.from_dict(_v), "attributes")
        if (_value := data.get("startTimeUnixNano")) is not None:
            _args["start_time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "start_time_unix_nano")
        if (_value := data.get("timeUnixNano")) is not None:
            _args["time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "time_unix_nano")
        if (_value := data.get("count")) is not None:
            _args["count"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "count")
        if (_value := data.get("sum")) is not None:
            _args["sum"] = opentelemetry.proto_json._json_codec.decode_float(_value, "sum")
        if (_value := data.get("bucketCounts")) is not None:
            _args["bucket_counts"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json._json_codec.decode_int64(_v, "bucket_counts"), "bucket_counts")
        if (_value := data.get("explicitBounds")) is not None:
            _args["explicit_bounds"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json._json_codec.decode_float(_v, "explicit_bounds"), "explicit_bounds")
        if (_value := data.get("exemplars")) is not None:
            _args["exemplars"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: Exemplar.from_dict(_v), "exemplars")
        if (_value := data.get("flags")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "flags")
            _args["flags"] = _value
        if (_value := data.get("min")) is not None:
            _args["min"] = opentelemetry.proto_json._json_codec.decode_float(_value, "min")
        if (_value := data.get("max")) is not None:
            _args["max"] = opentelemetry.proto_json._json_codec.decode_float(_value, "max")

        return cls(**_args)


@typing.final
@_dataclass
class ExponentialHistogramDataPoint(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ExponentialHistogramDataPoint
    """

    @typing.final
    @_dataclass
    class Buckets(opentelemetry.proto_json._json_codec.JsonMessage):
        """
        Generated from protobuf message Buckets
        """

        offset: builtins.int | None = 0
        bucket_counts: builtins.list[builtins.int] = dataclasses.field(default_factory=builtins.list)

        def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
            """
            Convert this message to a dictionary with lowerCamelCase keys.

            Returns:
                Dictionary representation following OTLP JSON encoding
            """
            _result = {}
            if self.offset:
                _result["offset"] = self.offset
            if self.bucket_counts:
                _result["bucketCounts"] = opentelemetry.proto_json._json_codec.encode_repeated(self.bucket_counts, lambda _v: opentelemetry.proto_json._json_codec.encode_int64(_v))
            return _result

        @builtins.classmethod
        def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ExponentialHistogramDataPoint.Buckets":
            """
            Create from a dictionary with lowerCamelCase keys.

            Args:
                data: Dictionary representation following OTLP JSON encoding

            Returns:
                Buckets instance
            """
            opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
            _args = {}

            if (_value := data.get("offset")) is not None:
                opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "offset")
                _args["offset"] = _value
            if (_value := data.get("bucketCounts")) is not None:
                _args["bucket_counts"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json._json_codec.decode_int64(_v, "bucket_counts"), "bucket_counts")

            return cls(**_args)

    attributes: builtins.list[opentelemetry.proto_json.common.v1.common.KeyValue] = dataclasses.field(default_factory=builtins.list)
    start_time_unix_nano: builtins.int | None = 0
    time_unix_nano: builtins.int | None = 0
    count: builtins.int | None = 0
    sum: builtins.float | None = None
    scale: builtins.int | None = 0
    zero_count: builtins.int | None = 0
    positive: ExponentialHistogramDataPoint.Buckets | None = None
    negative: ExponentialHistogramDataPoint.Buckets | None = None
    flags: builtins.int | None = 0
    exemplars: builtins.list[Exemplar] = dataclasses.field(default_factory=builtins.list)
    min: builtins.float | None = None
    max: builtins.float | None = None
    zero_threshold: builtins.float | None = 0.0

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.attributes:
            _result["attributes"] = opentelemetry.proto_json._json_codec.encode_repeated(self.attributes, lambda _v: _v.to_dict())
        if self.start_time_unix_nano:
            _result["startTimeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.start_time_unix_nano)
        if self.time_unix_nano:
            _result["timeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.time_unix_nano)
        if self.count:
            _result["count"] = opentelemetry.proto_json._json_codec.encode_int64(self.count)
        if self.sum is not None:
            _result["sum"] = opentelemetry.proto_json._json_codec.encode_float(self.sum)
        if self.scale:
            _result["scale"] = self.scale
        if self.zero_count:
            _result["zeroCount"] = opentelemetry.proto_json._json_codec.encode_int64(self.zero_count)
        if self.positive:
            _result["positive"] = self.positive.to_dict()
        if self.negative:
            _result["negative"] = self.negative.to_dict()
        if self.flags:
            _result["flags"] = self.flags
        if self.exemplars:
            _result["exemplars"] = opentelemetry.proto_json._json_codec.encode_repeated(self.exemplars, lambda _v: _v.to_dict())
        if self.min is not None:
            _result["min"] = opentelemetry.proto_json._json_codec.encode_float(self.min)
        if self.max is not None:
            _result["max"] = opentelemetry.proto_json._json_codec.encode_float(self.max)
        if self.zero_threshold:
            _result["zeroThreshold"] = opentelemetry.proto_json._json_codec.encode_float(self.zero_threshold)
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ExponentialHistogramDataPoint":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ExponentialHistogramDataPoint instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("attributes")) is not None:
            _args["attributes"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.common.v1.common.KeyValue.from_dict(_v), "attributes")
        if (_value := data.get("startTimeUnixNano")) is not None:
            _args["start_time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "start_time_unix_nano")
        if (_value := data.get("timeUnixNano")) is not None:
            _args["time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "time_unix_nano")
        if (_value := data.get("count")) is not None:
            _args["count"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "count")
        if (_value := data.get("sum")) is not None:
            _args["sum"] = opentelemetry.proto_json._json_codec.decode_float(_value, "sum")
        if (_value := data.get("scale")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "scale")
            _args["scale"] = _value
        if (_value := data.get("zeroCount")) is not None:
            _args["zero_count"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "zero_count")
        if (_value := data.get("positive")) is not None:
            _args["positive"] = ExponentialHistogramDataPoint.Buckets.from_dict(_value)
        if (_value := data.get("negative")) is not None:
            _args["negative"] = ExponentialHistogramDataPoint.Buckets.from_dict(_value)
        if (_value := data.get("flags")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "flags")
            _args["flags"] = _value
        if (_value := data.get("exemplars")) is not None:
            _args["exemplars"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: Exemplar.from_dict(_v), "exemplars")
        if (_value := data.get("min")) is not None:
            _args["min"] = opentelemetry.proto_json._json_codec.decode_float(_value, "min")
        if (_value := data.get("max")) is not None:
            _args["max"] = opentelemetry.proto_json._json_codec.decode_float(_value, "max")
        if (_value := data.get("zeroThreshold")) is not None:
            _args["zero_threshold"] = opentelemetry.proto_json._json_codec.decode_float(_value, "zero_threshold")

        return cls(**_args)


@typing.final
@_dataclass
class SummaryDataPoint(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message SummaryDataPoint
    """

    @typing.final
    @_dataclass
    class ValueAtQuantile(opentelemetry.proto_json._json_codec.JsonMessage):
        """
        Generated from protobuf message ValueAtQuantile
        """

        quantile: builtins.float | None = 0.0
        value: builtins.float | None = 0.0

        def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
            """
            Convert this message to a dictionary with lowerCamelCase keys.

            Returns:
                Dictionary representation following OTLP JSON encoding
            """
            _result = {}
            if self.quantile:
                _result["quantile"] = opentelemetry.proto_json._json_codec.encode_float(self.quantile)
            if self.value:
                _result["value"] = opentelemetry.proto_json._json_codec.encode_float(self.value)
            return _result

        @builtins.classmethod
        def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "SummaryDataPoint.ValueAtQuantile":
            """
            Create from a dictionary with lowerCamelCase keys.

            Args:
                data: Dictionary representation following OTLP JSON encoding

            Returns:
                ValueAtQuantile instance
            """
            opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
            _args = {}

            if (_value := data.get("quantile")) is not None:
                _args["quantile"] = opentelemetry.proto_json._json_codec.decode_float(_value, "quantile")
            if (_value := data.get("value")) is not None:
                _args["value"] = opentelemetry.proto_json._json_codec.decode_float(_value, "value")

            return cls(**_args)

    attributes: builtins.list[opentelemetry.proto_json.common.v1.common.KeyValue] = dataclasses.field(default_factory=builtins.list)
    start_time_unix_nano: builtins.int | None = 0
    time_unix_nano: builtins.int | None = 0
    count: builtins.int | None = 0
    sum: builtins.float | None = 0.0
    quantile_values: builtins.list[SummaryDataPoint.ValueAtQuantile] = dataclasses.field(default_factory=builtins.list)
    flags: builtins.int | None = 0

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.attributes:
            _result["attributes"] = opentelemetry.proto_json._json_codec.encode_repeated(self.attributes, lambda _v: _v.to_dict())
        if self.start_time_unix_nano:
            _result["startTimeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.start_time_unix_nano)
        if self.time_unix_nano:
            _result["timeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.time_unix_nano)
        if self.count:
            _result["count"] = opentelemetry.proto_json._json_codec.encode_int64(self.count)
        if self.sum:
            _result["sum"] = opentelemetry.proto_json._json_codec.encode_float(self.sum)
        if self.quantile_values:
            _result["quantileValues"] = opentelemetry.proto_json._json_codec.encode_repeated(self.quantile_values, lambda _v: _v.to_dict())
        if self.flags:
            _result["flags"] = self.flags
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "SummaryDataPoint":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            SummaryDataPoint instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("attributes")) is not None:
            _args["attributes"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.common.v1.common.KeyValue.from_dict(_v), "attributes")
        if (_value := data.get("startTimeUnixNano")) is not None:
            _args["start_time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "start_time_unix_nano")
        if (_value := data.get("timeUnixNano")) is not None:
            _args["time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "time_unix_nano")
        if (_value := data.get("count")) is not None:
            _args["count"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "count")
        if (_value := data.get("sum")) is not None:
            _args["sum"] = opentelemetry.proto_json._json_codec.decode_float(_value, "sum")
        if (_value := data.get("quantileValues")) is not None:
            _args["quantile_values"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: SummaryDataPoint.ValueAtQuantile.from_dict(_v), "quantile_values")
        if (_value := data.get("flags")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.int, "flags")
            _args["flags"] = _value

        return cls(**_args)


@typing.final
@_dataclass
class Exemplar(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message Exemplar
    """

    filtered_attributes: builtins.list[opentelemetry.proto_json.common.v1.common.KeyValue] = dataclasses.field(default_factory=builtins.list)
    time_unix_nano: builtins.int | None = 0
    as_double: builtins.float | None = None
    as_int: builtins.int | None = None
    span_id: builtins.bytes | None = b""
    trace_id: builtins.bytes | None = b""

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.filtered_attributes:
            _result["filteredAttributes"] = opentelemetry.proto_json._json_codec.encode_repeated(self.filtered_attributes, lambda _v: _v.to_dict())
        if self.time_unix_nano:
            _result["timeUnixNano"] = opentelemetry.proto_json._json_codec.encode_int64(self.time_unix_nano)
        if self.span_id:
            _result["spanId"] = opentelemetry.proto_json._json_codec.encode_hex(self.span_id)
        if self.trace_id:
            _result["traceId"] = opentelemetry.proto_json._json_codec.encode_hex(self.trace_id)
        if self.as_int is not None:
            _result["asInt"] = opentelemetry.proto_json._json_codec.encode_int64(self.as_int)
        elif self.as_double is not None:
            _result["asDouble"] = opentelemetry.proto_json._json_codec.encode_float(self.as_double)
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "Exemplar":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            Exemplar instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("filteredAttributes")) is not None:
            _args["filtered_attributes"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.common.v1.common.KeyValue.from_dict(_v), "filtered_attributes")
        if (_value := data.get("timeUnixNano")) is not None:
            _args["time_unix_nano"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "time_unix_nano")
        if (_value := data.get("spanId")) is not None:
            _args["span_id"] = opentelemetry.proto_json._json_codec.decode_hex(_value, "span_id")
        if (_value := data.get("traceId")) is not None:
            _args["trace_id"] = opentelemetry.proto_json._json_codec.decode_hex(_value, "trace_id")
        if (_value := data.get("asInt")) is not None:
            _args["as_int"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "as_int")
        elif (_value := data.get("asDouble")) is not None:
            _args["as_double"] = opentelemetry.proto_json._json_codec.decode_float(_value, "as_double")

        return cls(**_args)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from collections import Counter
from typing import Protocol

from opentelemetry.metrics import MeterProvider
from opentelemetry.semconv._incubating.attributes.otel_attributes import (
    OTEL_COMPONENT_NAME,
    OTEL_COMPONENT_TYPE,
)
from opentelemetry.semconv._incubating.metrics.otel_metrics import (
    create_otel_sdk_metric_reader_collection_duration,
)

_component_counter = Counter()


class MetricReaderMetricsT(Protocol):
    def record_collection(self, duration: float) -> None: ...


class NoOpMetricReaderMetrics:
    def record_collection(self, duration: float) -> None:
        pass


class MetricReaderMetrics:
    def __init__(
        self, component_type: str, meter_provider: MeterProvider
    ) -> None:
        meter = meter_provider.get_meter("opentelemetry-sdk")

        count = _component_counter[component_type]
        _component_counter[component_type] = count + 1

        self._standard_attrs = {
            OTEL_COMPONENT_TYPE: component_type,
            OTEL_COMPONENT_NAME: f"{component_type}/{count}",
        }

        self._collection_duration = (
            create_otel_sdk_metric_reader_collection_duration(meter)
        )

    def record_collection(self, duration: float) -> None:
        self._collection_duration.record(duration, self._standard_attrs)


def create_metric_reader_metrics(
    component_type: str,
    meter_provider: MeterProvider,
    enabled: bool,
) -> MetricReaderMetricsT:
    if not enabled:
        return NoOpMetricReaderMetrics()

    return MetricReaderMetrics(component_type, meter_provider)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

# AUTO-GENERATED from "opentelemetry/proto/collector/logs/v1/logs_service.proto"
# DO NOT EDIT MANUALLY

from __future__ import annotations

import builtins
import dataclasses
import functools
import typing

_dataclass = functools.partial(dataclasses.dataclass, slots=True)

import opentelemetry.proto_json._json_codec
import opentelemetry.proto_json.logs.v1.logs


@typing.final
@_dataclass
class ExportLogsServiceRequest(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ExportLogsServiceRequest
    """

    resource_logs: builtins.list[opentelemetry.proto_json.logs.v1.logs.ResourceLogs] = dataclasses.field(default_factory=builtins.list)

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.resource_logs:
            _result["resourceLogs"] = opentelemetry.proto_json._json_codec.encode_repeated(self.resource_logs, lambda _v: _v.to_dict())
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ExportLogsServiceRequest":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ExportLogsServiceRequest instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("resourceLogs")) is not None:
            _args["resource_logs"] = opentelemetry.proto_json._json_codec.decode_repeated(_value, lambda _v: opentelemetry.proto_json.logs.v1.logs.ResourceLogs.from_dict(_v), "resource_logs")

        return cls(**_args)


@typing.final
@_dataclass
class ExportLogsServiceResponse(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ExportLogsServiceResponse
    """

    partial_success: ExportLogsPartialSuccess | None = None

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.partial_success:
            _result["partialSuccess"] = self.partial_success.to_dict()
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ExportLogsServiceResponse":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ExportLogsServiceResponse instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("partialSuccess")) is not None:
            _args["partial_success"] = ExportLogsPartialSuccess.from_dict(_value)

        return cls(**_args)


@typing.final
@_dataclass
class ExportLogsPartialSuccess(opentelemetry.proto_json._json_codec.JsonMessage):
    """
    Generated from protobuf message ExportLogsPartialSuccess
    """

    rejected_log_records: builtins.int | None = 0
    error_message: builtins.str | None = ""

    def to_dict(self) -> builtins.dict[builtins.str, typing.Any]:
        """
        Convert this message to a dictionary with lowerCamelCase keys.

        Returns:
            Dictionary representation following OTLP JSON encoding
        """
        _result = {}
        if self.rejected_log_records:
            _result["rejectedLogRecords"] = opentelemetry.proto_json._json_codec.encode_int64(self.rejected_log_records)
        if self.error_message:
            _result["errorMessage"] = self.error_message
        return _result

    @builtins.classmethod
    def from_dict(cls, data: builtins.dict[builtins.str, typing.Any]) -> "ExportLogsPartialSuccess":
        """
        Create from a dictionary with lowerCamelCase keys.

        Args:
            data: Dictionary representation following OTLP JSON encoding

        Returns:
            ExportLogsPartialSuccess instance
        """
        opentelemetry.proto_json._json_codec.validate_type(data, builtins.dict, "data")
        _args = {}

        if (_value := data.get("rejectedLogRecords")) is not None:
            _args["rejected_log_records"] = opentelemetry.proto_json._json_codec.decode_int64(_value, "rejected_log_records")
        if (_value := data.get("errorMessage")) is not None:
            opentelemetry.proto_json._json_codec.validate_type(_value, builtins.str, "error_message")
            _args["error_message"] = _value

        return cls(**_args)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import TYPE_CHECKING, cast

# pylint: disable-next=import-error
from opentelemetry.exporter.http.transport._requests import (
    RequestsHTTPTransport as _RequestsHTTPTransport,
)

# pylint: disable-next=import-error
from opentelemetry.exporter.http.transport._urllib3 import (
    Urllib3HTTPTransport as _Urllib3HTTPTransport,
)

if TYPE_CHECKING:
    from typing import Any, Protocol

    from opentelemetry.exporter.http.transport._base import BaseHTTPTransport

    class BaseHTTPTransportFactory(Protocol):
        def __call__(
            self,
            *,
            verify: bool | str,
            cert: str | tuple[str, str] | None,
            **kwargs: Any,
        ) -> BaseHTTPTransport: ...


_KNOWN_TRANSPORTS: dict[str, BaseHTTPTransportFactory] = {
    "requests": _RequestsHTTPTransport,
    "urllib3": _Urllib3HTTPTransport,
}


def _load_http_transport_factory(name: str) -> BaseHTTPTransportFactory:
    """Return the transport factory registered under *name*.

    Checks the built-in transport registry first to avoid the overhead of an
    entry-point scan for built-in transports. Falls back to standard entry-point
    discovery for user supplied transports registered under the
    ``opentelemetry_http_transport`` group.

    :param name: Entry point name, e.g. ``"requests"`` or ``"urllib3"``.
    :returns: A callable with signature
        ``(*, verify, cert, **kwargs) -> BaseHTTPTransport``.
    :raises ValueError: If no transport is registered under *name*.
    :raises TypeError: If the loaded entry point is not callable.
    """
    if name in _KNOWN_TRANSPORTS:
        return _KNOWN_TRANSPORTS[name]
    # pylint: disable-next=import-outside-toplevel,import-error
    from opentelemetry.util._importlib_metadata import (  # noqa: PLC0415
        entry_points,
    )

    ep = next(
        iter(entry_points(group="opentelemetry_http_transport", name=name)),
        None,
    )
    if not ep:
        raise ValueError(
            f"No HTTP transport registered under name {name!r}. "
            "Install the corresponding extra or register an entry point "
            "under the 'opentelemetry_http_transport' group."
        )
    factory = ep.load()
    if not callable(factory):
        raise TypeError(
            f"Transport {name!r} loaded from entry point is not callable "
            f"(got {factory!r})."
        )
    return cast("BaseHTTPTransportFactory", factory)



# Copyright The OpenTelemetry Authors
# SPDX-License-Identifier: Apache-2.0

from typing import Final

EXCEPTION_ESCAPED: Final = "exception.escaped"
"""
Deprecated: It's no longer recommended to record exceptions that are handled and do not escape the scope of a span.
"""

EXCEPTION_MESSAGE: Final = "exception.message"
"""
The exception message.
Note: > [!WARNING]
>
> This attribute may contain sensitive information.
"""

EXCEPTION_STACKTRACE: Final = "exception.stacktrace"
"""
A stacktrace as a string in the natural representation for the language runtime. The representation is to be determined and documented by each language SIG.
"""

EXCEPTION_TYPE: Final = "exception.type"
"""
The type of the exception (its fully-qualified class name, if applicable). The dynamic type of the exception should be preferred over the static type in languages that support it.
Note: If the recorded exception type is a wrapper that is not meaningful for
failure classification, instrumentation MAY use the type of the inner
exception instead. For example, in Go, errors created with `fmt.Errorf`
using `%w` MAY be unwrapped when the wrapper type does not help
classify the failure.
"""
